# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-14 15:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0028_rename_question'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='maximum',
            field=models.FloatField(blank=True, help_text='Maximum value for this question.', null=True, verbose_name='Maximum'),
        ),
        migrations.AddField(
            model_name='question',
            name='minimum',
            field=models.FloatField(blank=True, help_text='Minimal value for this question.', null=True, verbose_name='Minimum'),
        ),
        migrations.AddField(
            model_name='question',
            name='step',
            field=models.FloatField(blank=True, help_text='Step in which the value for this question can be incremented/decremented.', null=True, verbose_name='Step'),
        ),
        migrations.AddField(
            model_name='question',
            name='verbose_name_de',
            field=models.CharField(blank=True, help_text='German name displayed for this question (e.g. Projekt).', max_length=256, verbose_name='Name (de)'),
        ),
        migrations.AddField(
            model_name='question',
            name='verbose_name_en',
            field=models.CharField(blank=True, help_text='English name displayed for this question (e.g. project).', max_length=256, verbose_name='Name (en)'),
        ),
        migrations.AddField(
            model_name='question',
            name='verbose_name_plural_de',
            field=models.CharField(blank=True, help_text='German plural name displayed for this question (e.g. Projekte).', max_length=256, verbose_name='Plural name (de)'),
        ),
        migrations.AddField(
            model_name='question',
            name='verbose_name_plural_en',
            field=models.CharField(blank=True, help_text='English plural name displayed for this question (e.g. projects).', max_length=256, verbose_name='Plural name (en)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='verbose_name_de',
            field=models.CharField(blank=True, help_text='German name displayed for this question (e.g. Projekt).', max_length=256, verbose_name='Name (de)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='verbose_name_en',
            field=models.CharField(blank=True, help_text='English name displayed for this question (e.g. project).', max_length=256, verbose_name='Name (en)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='verbose_name_plural_de',
            field=models.CharField(blank=True, help_text='German plural name displayed for this question (e.g. Projekte).', max_length=256, verbose_name='Plural name (de)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='verbose_name_plural_en',
            field=models.CharField(blank=True, help_text='English plural name displayed for this question (e.g. projects).', max_length=256, verbose_name='Plural name (en)'),
        ),
    ]



from urllib.parse import urlparse

from django import template

from rdmo.core.constants import (VALUE_TYPE_DATETIME, VALUE_TYPE_INTEGER,
                                 VALUE_TYPE_TEXT)
from rdmo.projects.models import Value

register = template.Library()


@register.simple_tag(takes_context=True)
def get_values(context, attribute, set_prefix='*', set_index='*', index='*', project=None):
    if project is None:
        project = context['project']

    if attribute == 'project/id':
        return [Value(text=project.id, value_type=VALUE_TYPE_INTEGER).as_dict]
    elif attribute == 'project/title':
        return [Value(text=project.title, value_type=VALUE_TYPE_TEXT).as_dict]
    elif attribute == 'project/description':
        return [Value(text=project.description, value_type=VALUE_TYPE_TEXT).as_dict]
    elif attribute == 'project/created':
        return [Value(text=project.created, value_type=VALUE_TYPE_DATETIME).as_dict]
    elif attribute == 'project/updated':
        return [Value(text=project.updated, value_type=VALUE_TYPE_DATETIME).as_dict]
    if attribute == 'project/snapshot/id':
        return [Value(text=project.snapshot.get('id'), value_type=VALUE_TYPE_INTEGER).as_dict]
    elif attribute == 'project/snapshot/title':
        return [Value(text=project.snapshot.get('title'), value_type=VALUE_TYPE_TEXT).as_dict]
    elif attribute == 'project/snapshot/description':
        return [Value(text=project.snapshot.get('description'), value_type=VALUE_TYPE_TEXT).as_dict]
    elif attribute == 'project/snapshot/created':
        return [Value(text=project.snapshot.get('created'), value_type=VALUE_TYPE_DATETIME).as_dict]
    elif attribute == 'project/snapshot/updated':
        return [Value(text=project.snapshot.get('updated'), value_type=VALUE_TYPE_DATETIME).as_dict]
    else:
        values = project._values

        if urlparse(attribute).scheme:
            values = filter(lambda value: value.attribute and (value.attribute.uri == attribute), values)
        else:
            values = filter(lambda value: value.attribute and (value.attribute.path == attribute), values)

        if set_prefix != '*':
            values = filter(lambda value: value.set_prefix == set_prefix, values)

        if set_index != '*':
            values = filter(lambda value: value.set_index == set_index, values)

        if index != '*':
            values = filter(lambda value: value.collection_index == index, values)

        return list(map(lambda value: value.as_dict, values))


@register.simple_tag(takes_context=True)
def get_numbers(context, attribute, set_prefix='*', set_index='*', index='*', project=None):
    values = get_values(context, attribute, set_prefix, set_index, index, project)
    return [value.get('as_number', 0) for value in values]


@register.simple_tag(takes_context=True)
def get_value(context, attribute, set_prefix='', set_index=0, index=0, project=None):
    try:
        return get_values(context, attribute, set_prefix=set_prefix, set_index=set_index, index=index, project=project)[0]
    except IndexError:
        return None


@register.simple_tag(takes_context=True)
def get_number(context, attribute, set_prefix='', set_index=0, index=0, project=None):
    value = get_value(context, attribute, set_prefix, set_index, index, project)
    if value is None:
        return 0
    else:
        return value.get('as_number', 0)


@register.simple_tag(takes_context=True)
def get_set_values(context, set, attribute, set_prefix='', project=None):
    set_index = set.get('set_index')
    return get_values(context, attribute, set_prefix=set_prefix, set_index=set_index, project=project)


@register.simple_tag(takes_context=True)
def get_set_value(context, set, attribute, set_prefix='', index=0, project=None):
    try:
        set_index = set.get('set_index')
        return get_values(context, attribute, set_prefix=set_prefix, set_index=set_index, index=index, project=project)[0]
    except IndexError:
        return None


@register.simple_tag(takes_context=True)
def get_set_prefixes(context, attribute, project=None):
    try:
        return sorted(set(map(lambda value: value['set_prefix'], get_values(context, attribute, project=project))))
    except IndexError:
        return None


@register.simple_tag(takes_context=True)
def get_set_indexes(context, attribute, set_prefix='', project=None):
    try:
        return sorted(set(map(lambda value: value['set_index'], get_values(context, attribute, set_prefix=set_prefix, project=project))))
    except IndexError:
        return None


@register.simple_tag(takes_context=True)
def get_sets(context, attribute, set_prefix='', project=None):
    # get the values for the set attribute
    values = get_values(context, attribute.rstrip('/'), set_prefix=set_prefix, index=0, project=project)
    if values:
        return values
    elif not attribute.endswith('/id'):
        # for backwards compatibility, try again with the /id attribute
        return get_sets(context, attribute.rstrip('/') + '/id', set_prefix=set_prefix, project=project)


@register.simple_tag(takes_context=True)
def get_set(context, attribute, set_prefix='', project=None):
    # for backwards compatibility, identical to get_sets
    return get_sets(context, attribute, set_prefix=set_prefix, project=project)


@register.inclusion_tag('views/tags/value.html', takes_context=True)
def render_value(context, attribute, set_prefix='', set_index=0, index=0, project=None):
    context['value'] = get_value(context, attribute, set_prefix=set_prefix, set_index=set_index, index=index, project=project)
    return context


@register.inclusion_tag('views/tags/value_list.html', takes_context=True)
def render_value_list(context, attribute, set_prefix='', set_index=0, project=None):
    context['values'] = get_values(context, attribute, set_prefix=set_prefix, set_index=set_index, project=project)
    return context


@register.inclusion_tag('views/tags/value_inline_list.html', takes_context=True)
def render_value_inline_list(context, attribute, set_prefix='', set_index=0, project=None):
    context['values'] = get_values(context, attribute, set_prefix=set_prefix, set_index=set_index, project=project)
    return context


@register.inclusion_tag('views/tags/value.html', takes_context=True)
def render_set_value(context, set, attribute, set_prefix='', index=0, project=None):
    context['value'] = get_set_value(context, set, attribute, set_prefix=set_prefix, index=index, project=project)
    return context


@register.inclusion_tag('views/tags/value_list.html', takes_context=True)
def render_set_value_list(context, set, attribute, set_prefix='', project=None):
    context['values'] = get_set_values(context, set, attribute, set_prefix=set_prefix, project=project)
    return context


@register.inclusion_tag('views/tags/value_inline_list.html', takes_context=True)
def render_set_value_inline_list(context, set, attribute, set_prefix='', project=None):
    context['values'] = get_set_values(context, set, attribute, set_prefix=set_prefix, project=project)
    return context


@register.simple_tag(takes_context=True)
def check_condition(context, condition, set_prefix=None, set_index=None, project=None):
    if project is None:
        project = context['project']

    conditions = project._conditions
    if urlparse(condition).scheme:
        conditions = filter(lambda c: c.uri == condition, conditions)
    else:
        conditions = filter(lambda c: c.key == condition, conditions)

    return project._check_conditions(conditions, set_prefix=set_prefix, set_index=set_index)


@register.filter
def is_true(values):
    return [value for value in values if value['is_true']]


@register.filter
def is_false(values):
    return [value for value in values if value['is_false']]


@register.filter
def is_empty(values):
    return [value for value in values if value['is_empty']]


@register.filter
def is_not_empty(values):
    return [value for value in values if not value['is_empty']]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-30 14:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0039_meta'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this attribute (auto-generated).', max_length=640, null=True, verbose_name='URI'),
        ),
    ]



import pytest
from django.core import mail
from django.urls import reverse

from ..models import Invite, Membership

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

add_membership_permission_map = change_membership_permission_map = delete_membership_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

projects = [1, 2, 3, 4, 5]
memberships = [1, 2, 3, 4]

membership_roles = ('owner', 'manager', 'author', 'guest')


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_membership_create_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('membership_create', args=[project_id])
    response = client.get(url)

    if project_id in add_membership_permission_map.get(username, []):
        assert response.status_code == 200
    elif password:
        assert response.status_code == 403
    else:
        assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('membership_role', membership_roles)
def test_membership_create_post(db, client, username, password, project_id, membership_role):
    client.login(username=username, password=password)

    url = reverse('membership_create', args=[project_id])
    data = {
        'username_or_email': 'user',
        'role': membership_role
    }
    response = client.post(url, data)

    if project_id in add_membership_permission_map.get(username, []):
        assert response.status_code == 302
        assert Invite.objects.get(project_id=project_id, user__username='user', email='user@example.com', role=membership_role)
        assert not Membership.objects.filter(project_id=project_id, user__username='user', role=membership_role).exists()
        assert len(mail.outbox) == 1
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert not Invite.objects.exists()
        assert len(mail.outbox) == 0


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('membership_role', membership_roles)
def test_membership_create_post_mail(db, client, username, password, project_id, membership_role):
    client.login(username=username, password=password)

    url = reverse('membership_create', args=[project_id])
    data = {
        'username_or_email': 'someuser@example.com',
        'role': membership_role
    }
    response = client.post(url, data)

    if project_id in add_membership_permission_map.get(username, []):
        assert response.status_code == 302
        assert Invite.objects.get(project_id=project_id, user=None, email='someuser@example.com', role=membership_role)
        assert len(mail.outbox) == 1
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert not Invite.objects.exists()
        assert len(mail.outbox) == 0


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('membership_role', membership_roles)
def test_membership_create_post_error(db, client, username, password, membership_role):
    client.login(username=username, password=password)

    project_id = 1
    url = reverse('membership_create', args=[project_id])
    data = {
        'username_or_email': 'guest',
        'role': membership_role
    }
    response = client.post(url, data)

    assert not Invite.objects.exists(), Invite.objects.all()
    assert len(mail.outbox) == 0

    if project_id in add_membership_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('membership_role', membership_roles)
def test_membership_create_post_mail_error(db, client, username, password, membership_role):
    client.login(username=username, password=password)

    project_id = 1
    url = reverse('membership_create', args=[project_id])
    data = {
        'username_or_email': 'guest@example.com',
        'role': membership_role
    }
    response = client.post(url, data)

    assert not Invite.objects.exists(), Invite.objects.all()
    assert len(mail.outbox) == 0

    if project_id in add_membership_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('membership_role', membership_roles)
def test_membership_create_post_silent(db, client, username, password, membership_role):
    client.login(username=username, password=password)

    project_id = 1
    url = reverse('membership_create', args=[project_id])
    data = {
        'username_or_email': 'user',
        'role': membership_role,
        'silent': True
    }
    response = client.post(url, data)

    if project_id in add_membership_permission_map.get(username, []):
        assert response.status_code == 302

        if username == 'site':
            assert not Invite.objects.exists()
            assert Membership.objects.get(project_id=project_id, user__username='user', role=membership_role)
            assert len(mail.outbox) == 0
        else:
            assert Invite.objects.get(project_id=project_id, user__username='user', email='user@example.com', role=membership_role)
            assert not Membership.objects.filter(project_id=project_id, user__username='user', role=membership_role).exists()
            assert len(mail.outbox) == 1

    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('membership_id', memberships)
def test_membership_update_get(db, client, username, password, project_id, membership_id):
    client.login(username=username, password=password)
    membership = Membership.objects.filter(project_id=project_id, id=membership_id).first()

    url = reverse('membership_update', args=[project_id, membership_id])
    response = client.get(url)

    if membership:
        if project_id in change_membership_permission_map.get(username, []):
            assert response.status_code == 200
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('membership_id', memberships)
@pytest.mark.parametrize('membership_role', membership_roles)
def test_membership_update_post(db, client, username, password, project_id, membership_id, membership_role):
    client.login(username=username, password=password)
    membership = Membership.objects.filter(project_id=project_id, id=membership_id).first()

    url = reverse('membership_update', args=[project_id, membership_id])
    data = {
        'role': membership_role
    }
    response = client.post(url, data)

    if membership:
        if project_id in change_membership_permission_map.get(username, []):
            assert response.status_code == 302
            assert Membership.objects.get(project_id=project_id, id=membership_id).role == membership_role

        else:
            if password:
                assert response.status_code == 403
            else:
                assert response.status_code == 302

            assert Membership.objects.get(project_id=project_id, id=membership_id).role == membership.role
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('membership_id', memberships)
def test_membership_delete_get(db, client, username, password, project_id, membership_id):
    client.login(username=username, password=password)
    membership = Membership.objects.filter(project_id=project_id, id=membership_id).first()

    url = reverse('membership_delete', args=[project_id, membership_id])
    response = client.get(url)

    if membership:
        if project_id in delete_membership_permission_map.get(username, []):
            assert response.status_code == 200
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('membership_id', memberships)
def test_membership_delete_post(db, client, mocker, username, password, project_id, membership_id):
    client.login(username=username, password=password)
    membership = Membership.objects.filter(project_id=project_id, id=membership_id).first()

    url = reverse('membership_delete', args=[project_id, membership_id])
    response = client.post(url)

    if membership:
        if project_id in delete_membership_permission_map.get(username, []):
            if membership.user.username == 'owner':
                assert response.status_code == 400
            else:
                assert response.status_code == 302
                assert not Membership.objects.filter(project_id=project_id, id=membership_id).first()
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-11-07 13:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0022_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attributeentity',
            name='label',
            field=models.CharField(db_index=True, max_length=512),
        ),
    ]



import logging

from django.contrib.sites.models import Site

from rdmo.core.imports import (get_m2m_instances, set_common_fields,
                               set_lang_field, validate_instance)
from rdmo.questions.models import Catalog

from .models import View
from .validators import ViewLockedValidator, ViewUniqueURIValidator

logger = logging.getLogger(__name__)


def import_view(element, save=False):
    try:
        view = View.objects.get(uri=element.get('uri'))
    except View.DoesNotExist:
        view = View()

    set_common_fields(view, element)

    view.template = element.get('template')

    set_lang_field(view, 'title', element)
    set_lang_field(view, 'help', element)

    catalogs = get_m2m_instances(view, element.get('catalogs'), Catalog)

    if save and validate_instance(view, ViewLockedValidator, ViewUniqueURIValidator):
        if view.id:
            logger.info('View created with uri %s.', element.get('uri'))
        else:
            logger.info('View %s updated.', element.get('uri'))

        view.save()
        view.sites.add(Site.objects.get_current())
        view.catalogs.set(catalogs)
        view.imported = True

    return view



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-13 11:22
from __future__ import unicode_literals

from django.db import migrations


def set_null_to_blank(queryset, fields):
    for element in queryset:
        for field in fields:
            value = getattr(element, field)
            if value is None:
                setattr(element, field, '')
        element.save()


def run_data_migration(apps, schema_editor):
    OptionSet = apps.get_model('options', 'OptionSet')
    Option = apps.get_model('options', 'Option')

    set_null_to_blank(OptionSet.objects.all(), [
        'uri',
        'uri_prefix',
        'key',
        'comment',
    ])
    set_null_to_blank(Option.objects.all(), [
        'uri',
        'uri_prefix',
        'key',
        'path',
        'comment',
        'text_lang1',
        'text_lang2',
        'text_lang3',
        'text_lang4',
        'text_lang5',
    ])

class Migration(migrations.Migration):

    dependencies = [
        ('options', '0017_add_language_fields'),
    ]

    operations = [
        migrations.RunPython(run_data_migration),
    ]



# Generated by Django 2.2.16 on 2020-12-04 12:46

from __future__ import unicode_literals

import django.db.models.deletion
import mptt.fields
from django.db import migrations, models
from mptt import managers, register


def rebuild_tree(apps, schema_editor):
    Project = apps.get_model('projects', 'Project')

    manager = managers.TreeManager()
    manager.model = Project

    register(Project)

    manager.contribute_to_class(Project, 'objects')
    manager.rebuild()


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0045_value_file'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='level',
            field=models.PositiveIntegerField(default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='project',
            name='lft',
            field=models.PositiveIntegerField(default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='project',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, help_text='The parent project of this project.', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='children', to='projects.Project', verbose_name='Parent project'),
        ),
        migrations.AddField(
            model_name='project',
            name='rght',
            field=models.PositiveIntegerField(default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='project',
            name='tree_id',
            field=models.PositiveIntegerField(db_index=True, default=0, editable=False),
            preserve_default=False,
        ),
        migrations.AlterModelOptions(
            name='project',
            options={'ordering': ('tree_id', 'level', 'title'), 'verbose_name': 'Project', 'verbose_name_plural': 'Projects'},
        ),
        migrations.RunPython(rebuild_tree),
    ]



from django.contrib.auth.mixins import LoginRequiredMixin

from rest_framework.response import Response
from rest_framework.schemas import SchemaGenerator
from rest_framework.views import APIView
from rest_framework_swagger import renderers

from .urls.v1 import urlpatterns


class SwaggerSchemaView(LoginRequiredMixin, APIView):
    renderer_classes = [
        renderers.SwaggerUIRenderer,
    ]

    def get(self, request):
        generator = SchemaGenerator(
            title="RDMO API",
            patterns=urlpatterns,
            url=request.path
        )
        schema = generator.get_schema(request=request)
        return Response(schema)



import logging
from pathlib import Path

logger = logging.getLogger(__name__)


def get_value_path(project, snapshot=None):
    if snapshot is None:
        return Path('projects') / str(project.id) / 'values'
    else:
        return Path('projects') / str(project.id) / 'snapshots' / str(snapshot.id) / 'values'


def is_last_owner(project, user):
    # check if user is owner
    if user in project.owners:
        # check if the user is the last owner
        return project.owners.count() <= 1
    else:
        return False


def check_conditions(conditions, values, set_prefix=None, set_index=None):
    if conditions:
        for condition in conditions:
            if condition.resolve(values, set_prefix, set_index):
                return True
        return False
    else:
        return True


def save_import_values(project, values, checked):
    for value in values:
        if value.attribute:
            value_key = '{value.attribute.uri}[{value.set_prefix}][{value.set_index}][{value.collection_index}]'.format(
                value=value
            )

            if value_key in checked:
                current_value = value.current
                if current_value is None:
                    # assert that this is a new value
                    assert value.pk is None

                    value.project = project
                    value.save()

                    if value.file:
                        value.copy_file(value.file_name, value.file)
                    else:
                        try:
                            name = value.file_import.get('name')
                            file = value.file_import.get('file')
                            value.file.save(name, file)
                        except AttributeError:
                            pass

                else:
                    # make sure we have the correct value
                    assert current_value.snapshot is None
                    assert current_value.attribute == value.attribute
                    assert current_value.set_prefix == value.set_prefix
                    assert current_value.set_index == value.set_index
                    assert current_value.collection_index == value.collection_index

                    # assert that this is an new value
                    assert current_value.pk is not None

                    current_value.text = value.text
                    current_value.option = value.option
                    current_value.value_type = value.value_type
                    current_value.unit = value.unit
                    current_value.save()

                    if value.file:
                        current_value.file.delete()
                        current_value.copy_file(value.file_name, value.file)
                    else:
                        try:
                            name = value.file_import.get('name')
                            file = value.file_import.get('file')
                            current_value.file.delete()
                            current_value.file.save(name, file)
                        except AttributeError:
                            pass


def save_import_snapshot_values(project, snapshots, checked):
    for snapshot in snapshots:
        # assert that this is a new snapshot
        assert snapshot.pk is None

        snapshot.project = project
        snapshot.save(copy_values=False)

        for value in snapshot.snapshot_values:
            if value.attribute:
                value_key = '{value.attribute.uri}[{snapshot_index}][{value.set_prefix}][{value.set_index}][{value.collection_index}]'.format(
                    value=value,
                    snapshot_index=snapshot.snapshot_index
                )

                if value_key in checked:
                    # assert that this is a new value
                    assert value.pk is None

                    value.project = project
                    value.snapshot = snapshot
                    value.save()

                    if value.file:
                        value.copy_file(value.file_name, value.file)
                    else:
                        try:
                            name = value.file_import.get('name')
                            file = value.file_import.get('file')
                            value.file.save(name, file)
                        except AttributeError:
                            pass


def save_import_tasks(project, tasks):
    for task in tasks:
        project.tasks.add(task)


def save_import_views(project, views):
    for view in views:
        project.views.add(view)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-13 11:22
from __future__ import unicode_literals

from django.db import migrations


def set_null_to_blank(queryset, fields):
    for element in queryset:
        for field in fields:
            value = getattr(element, field)
            if value is None:
                setattr(element, field, '')
        element.save()


def run_data_migration(apps, schema_editor):
    View = apps.get_model('views', 'View')

    set_null_to_blank(View.objects.all(), [
        'uri',
        'uri_prefix',
        'key',
        'comment',
        'template',
        'title_lang1',
        'title_lang2',
        'title_lang3',
        'title_lang4',
        'title_lang5',
        'help_lang1',
        'help_lang2',
        'help_lang3',
        'help_lang4',
        'help_lang5',
    ])


class Migration(migrations.Migration):

    dependencies = [
        ('views', '0013_add_language_fields'),
    ]

    operations = [
        migrations.RunPython(run_data_migration),
    ]



import re

from setuptools import find_packages, setup

# get metadata from mudule using a regexp
with open('rdmo/__init__.py') as f:
    metadata = dict(re.findall(r'__(.*)__ = [\']([^\']*)[\']', f.read()))

# get install_requires from requirements.txt
with open('requirements.txt') as f:
    install_requires = f.readlines()

setup(
    name=metadata['title'],
    version=metadata['version'],
    author=metadata['author'],
    author_email=metadata['email'],
    maintainer=metadata['author'],
    maintainer_email=metadata['email'],
    license=metadata['license'],
    url='https://github.com/rdmorganiser/rdmo',
    description=u'RDMO is a tool to support the systematic planning, organisation and implementation of the data management throughout the course of a research project.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.5'
    ],
    packages=find_packages(),
    include_package_data=True
)



from django.core.management.base import BaseCommand

from rdmo.conditions.models import Condition
from rdmo.options.models import OptionSet, Option
from rdmo.domain.models import Attribute
from rdmo.questions.models import Catalog, Section, Subsection, QuestionSet, Question
from rdmo.tasks.models import Task
from rdmo.views.models import View


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('uri_prefix', action='store', help='URI prefix to be used for all elements.')

    def handle(self, *args, **options):

        for obj in Condition.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

        for obj in OptionSet.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

        for obj in Option.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

        for obj in Attribute.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

        for obj in Catalog.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

        for obj in Section.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

        for obj in Subsection.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

        for obj in QuestionSet.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

        for obj in Question.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

        for obj in Task.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

        for obj in View.objects.all():
            self._set_uri_prefix(obj, options['uri_prefix'])

    def _set_uri_prefix(self, obj, uri_prefix):
        obj.uri_prefix = uri_prefix
        obj.save()



# Generated by Django 2.2.18 on 2021-05-27 11:40

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0061_question_width'),
    ]

    operations = [
        migrations.AlterField(
            model_name='questionset',
            name='attribute',
            field=models.ForeignKey(blank=True, help_text='The attribute this question set belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='questionsets', to='domain.Attribute', verbose_name='Attribute'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional internal information about this question set.', verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='conditions',
            field=models.ManyToManyField(blank=True, help_text='List of conditions evaluated for this question set.', related_name='questionsets', to='conditions.Condition', verbose_name='Conditions'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='help_lang1',
            field=models.TextField(blank=True, help_text='The help text for this question set in the primary language.', verbose_name='Help (primary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='help_lang2',
            field=models.TextField(blank=True, help_text='The help text for this question set in the secondary language.', verbose_name='Help (secondary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='help_lang3',
            field=models.TextField(blank=True, help_text='The help text for this question set in the tertiary language.', verbose_name='Help (tertiary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='help_lang4',
            field=models.TextField(blank=True, help_text='The help text for this question set in the quaternary language.', verbose_name='Help (quaternary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='help_lang5',
            field=models.TextField(blank=True, help_text='The help text for this question set in the quinary language.', verbose_name='Help (quinary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='is_collection',
            field=models.BooleanField(default=False, help_text='Designates whether this question set is a collection.', verbose_name='is collection'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this question set.', max_length=128, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='locked',
            field=models.BooleanField(default=False, help_text="Designates whether this question set (and it's questions) can be changed.", verbose_name='Locked'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='order',
            field=models.IntegerField(default=0, help_text='The position of this question set in lists.', verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='path',
            field=models.CharField(blank=True, help_text='The path part of the URI of this question set (auto-generated).', max_length=512, verbose_name='Path'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='section',
            field=models.ForeignKey(help_text='The section this question set belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='questionsets', to='questions.Section', verbose_name='Section'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='title_lang1',
            field=models.CharField(blank=True, help_text='The title for this question set in the primary language.', max_length=256, verbose_name='Title (primary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='title_lang2',
            field=models.CharField(blank=True, help_text='The title for this question set in the secondary language.', max_length=256, verbose_name='Title (secondary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='title_lang3',
            field=models.CharField(blank=True, help_text='The title for this question set in the tertiary language.', max_length=256, verbose_name='Title (tertiary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='title_lang4',
            field=models.CharField(blank=True, help_text='The title for this question set in the quaternary language.', max_length=256, verbose_name='Title (quaternary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='title_lang5',
            field=models.CharField(blank=True, help_text='The title for this question set in the quinary language.', max_length=256, verbose_name='Title (quinary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this question set (auto-generated).', max_length=640, verbose_name='URI'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='uri_prefix',
            field=models.URLField(help_text='The prefix for the URI of this question set.', max_length=256, verbose_name='URI Prefix'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_lang1',
            field=models.CharField(blank=True, help_text='The name displayed for this question set in the primary language.', max_length=256, verbose_name='Name (primary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_lang2',
            field=models.CharField(blank=True, help_text='The name displayed for this question set in the secondary language.', max_length=256, verbose_name='Name (secondary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_lang3',
            field=models.CharField(blank=True, help_text='The name displayed for this question set in the tertiary language.', max_length=256, verbose_name='Name (tertiary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_lang4',
            field=models.CharField(blank=True, help_text='The name displayed for this question set in the quaternary language.', max_length=256, verbose_name='Name (quaternary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_lang5',
            field=models.CharField(blank=True, help_text='The name displayed for this question set in the quinary language.', max_length=256, verbose_name='Name (quinary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_plural_lang1',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question set in the primary language.', max_length=256, verbose_name='Plural name (primary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_plural_lang2',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question set in the secondary language.', max_length=256, verbose_name='Plural name (secondary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_plural_lang3',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question set in the tertiary language.', max_length=256, verbose_name='Plural name (tertiary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_plural_lang4',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question set in the quaternary language.', max_length=256, verbose_name='Plural name (quaternary)'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='verbose_name_plural_lang5',
            field=models.CharField(blank=True, help_text='The plural name displayed for this question set in the quinary language.', max_length=256, verbose_name='Plural name (quinary)'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-16 14:12
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0035_remove_is_collection_and_parent_collection'),
        ('questions', '0030_data_migration')
    ]

    operations = [
        migrations.RemoveField(
            model_name='range',
            name='attribute',
        ),
        migrations.RemoveField(
            model_name='verbosename',
            name='attribute_entity',
        ),
        migrations.DeleteModel(
            name='Range',
        ),
        migrations.DeleteModel(
            name='VerboseName',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-27 15:24
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0008_data_migration'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='questionentity',
            options={'ordering': ('subsection__section__catalog__order', 'subsection__section__order', 'subsection__order', 'order'), 'verbose_name': 'Question entity', 'verbose_name_plural': 'Question entities'},
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='attribute_entity',
            field=models.ForeignKey(blank=True, help_text='The attribute/entity this question/questionset belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.AttributeEntity', verbose_name='Attribute entity'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='order',
            field=models.IntegerField(default=0, help_text='The position of this question/questionset in lists.', verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='subsection',
            field=models.ForeignKey(help_text='The section this question/questionset belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='entities', to='questions.Subsection', verbose_name='Subsection'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='section',
            field=models.ForeignKey(help_text='The section this subsection belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='subsections', to='questions.Section', verbose_name='Section'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-17 11:49

import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('projects', '0026_django2'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='project',
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('on_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.AddField(
            model_name='project',
            name='site',
            field=models.ForeignKey(default=1, help_text='The site this view belongs to (in a multi site setup).', on_delete=django.db.models.deletion.CASCADE, to='sites.Site', verbose_name='Site'),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.2.16 on 2020-10-19 15:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0018_blank_fields'),
    ]

    operations = [
        migrations.DeleteModel(
            name='ProxyUser',
        ),
    ]



from django.urls import re_path

from ..views import OptionsExportView, OptionsView

urlpatterns = [
    re_path(r'^$', OptionsView.as_view(), name='options'),
    re_path(r'^export/(?P<format>[a-z]+)/$', OptionsExportView.as_view(), name='options_export'),
]



# Generated by Django 2.2.16 on 2021-01-22 19:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0025_optionset_provider_key'),
    ]

    operations = [
        migrations.AddField(
            model_name='option',
            name='locked',
            field=models.BooleanField(default=False, help_text='Designates whether this option can be changed.', verbose_name='Locked'),
        ),
        migrations.AddField(
            model_name='optionset',
            name='locked',
            field=models.BooleanField(default=False, help_text="Designates whether this option set (and it's options) can be changed.", verbose_name='Locked'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-08-09 12:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0016_label'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='value_type',
            field=models.CharField(choices=[('text', 'Text'), ('url', 'URL'), ('integer', 'Integer'), ('float', 'Float'), ('boolean', 'Boolean'), ('datetime', 'Datetime'), ('options', 'Options')], max_length=8),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-11-29 11:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0033_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='questionset',
            name='section',
            field=models.ForeignKey(help_text='The section this questionset belongs to.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='questionsets', to='questions.Section', verbose_name='Section'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='title_de',
            field=models.CharField(help_text='The German title for this subsection.', max_length=256, null=True, verbose_name='Title (de)'),
        ),
        migrations.AddField(
            model_name='questionset',
            name='title_en',
            field=models.CharField(help_text='The English title for this subsection.', max_length=256, null=True, verbose_name='Title (en)'),
        ),
    ]



from django import forms
from django.contrib import admin

from rdmo.core.utils import get_language_fields

from .models import View
from .validators import ViewLockedValidator, ViewUniqueURIValidator


class ViewAdminForm(forms.ModelForm):
    key = forms.SlugField(required=True)

    class Meta:
        model = View
        fields = '__all__'

    def clean(self):
        ViewUniqueURIValidator(self.instance)(self.cleaned_data)
        ViewLockedValidator(self.instance)(self.cleaned_data)


class ViewAdmin(admin.ModelAdmin):
    form = ViewAdminForm

    search_fields = ['uri'] + get_language_fields('title') + get_language_fields('help')
    list_display = ('uri', 'title', 'help', 'available')
    readonly_fields = ('uri', )
    list_filter = ('available', )


admin.site.register(View, ViewAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-25 13:56
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0010_refactoring'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='condition',
            options={'ordering': ('uri',), 'verbose_name': 'Condition', 'verbose_name_plural': 'Conditions'},
        ),
        migrations.AddField(
            model_name='condition',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this option set (auto-generated).', max_length=640, null=True, verbose_name='URI'),
        ),
        migrations.AddField(
            model_name='condition',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this condition.', max_length=256, null=True, verbose_name='URI Prefix'),
        ),
        migrations.AlterField(
            model_name='condition',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional information about this condition.', null=True, verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='condition',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this condition. The URI will be generated from this key.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='condition',
            name='relation',
            field=models.CharField(choices=[('eq', 'is equal to (==)'), ('neq', 'is not equal to (!=)'), ('contains', 'contains'), ('gt', 'is greater than (>)'), ('gte', 'is greater than or equal (>=)'), ('lt', 'is lesser than (<)'), ('lte', 'is lesser than or equal (<=)'), ('empty', 'is empty'), ('notempty', 'is not empty')], help_text='Relation this condition is using.', max_length=8, verbose_name='Relation'),
        ),
        migrations.AlterField(
            model_name='condition',
            name='source',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='Attribute this condition is evaluating.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Attribute', verbose_name='Source'),
        ),
        migrations.AlterField(
            model_name='condition',
            name='target_option',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='Option this condition is checking against.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='options.Option', verbose_name='Target (Option)'),
        ),
        migrations.AlterField(
            model_name='condition',
            name='target_text',
            field=models.CharField(blank=True, help_text='Raw text value this condition is checking against.', max_length=256, null=True, verbose_name='Target (Text)'),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations


def run_data_migration(apps, schema_editor):
    Project = apps.get_model('projects', 'Project')
    Issue = apps.get_model('projects', 'Issue')

    for project in Project.objects.all():
        # add issues for all tasks to project
        for task in project.tasks.all():
            issue = Issue(project=project, task=task)
            issue.save()


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0034_issue'),
        ('tasks', '0031_related_name')
    ]

    operations = [
        migrations.RunPython(run_data_migration),
    ]



import pytest
from django.urls import reverse

from ..models import Membership

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

leave_project_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3],
    'author': [1, 3],
    'guest': [1, 3]
}

projects = [1, 2, 3, 4, 5]


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_leave_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_leave', args=[project_id])
    response = client.get(url)

    if project_id in leave_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_leave_post(db, client, username, password, project_id):
    client.login(username=username, password=password)

    # add additional owner, so that the owner can leave
    Membership.objects.create(user_id=1, project_id=project_id, role='owner')

    # store if the membership exists
    membership_existed = Membership.objects.filter(project_id=project_id, user__username=username).exists()

    url = reverse('project_leave', args=[project_id])
    response = client.post(url)

    if project_id in leave_project_permission_map.get(username, []):
        assert response.status_code == 302
        if membership_existed:
            assert not Membership.objects.filter(project_id=project_id, user__username=username).exists()
    else:
        if membership_existed:
            assert Membership.objects.filter(project_id=project_id, user__username=username).exists()

        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_leave_post_last_owner(db, client, username, password, project_id):
    client.login(username=username, password=password)

    # store if the membership exists
    membership_existed = Membership.objects.filter(project_id=project_id, user__username=username).exists()

    url = reverse('project_leave', args=[project_id])
    response = client.post(url)

    if project_id in leave_project_permission_map.get(username, []):
        assert response.status_code == 302

        if username == 'owner':
            if membership_existed:
                assert Membership.objects.filter(project_id=project_id, user__username=username).exists()
        else:
            if membership_existed:
                assert not Membership.objects.filter(project_id=project_id, user__username=username).exists()
    else:
        if membership_existed:
            assert Membership.objects.filter(project_id=project_id, user__username=username).exists()

        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_leave_post_cancel(db, client, username, password, project_id):
    client.login(username=username, password=password)

    # add additional owner, so that the owner can leave
    Membership.objects.create(user_id=1, project_id=project_id, role='owner')

    # store if the membership exists
    membership_existed = Membership.objects.filter(project_id=project_id, user__username=username).exists()

    url = reverse('project_leave', args=[project_id])
    response = client.post(url, {'cancel': True})

    if membership_existed:
        assert Membership.objects.filter(project_id=project_id, user__username=username).exists()

    if project_id in leave_project_permission_map.get(username, []):
        assert response.status_code == 302
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


# @pytest.mark.parametrize('username,password', users)
# @pytest.mark.parametrize('project_id', projects)
# def test_project_leave_post(db, client, username, password, project_id):
#     client.login(username=username, password=password)

#     # add additional owner, so that the owner can leave
#     Membership.objects.create(user_id=1, project_id=project_id, role='owner')

#     url = reverse('project_leave', args=[project_id])
#     response = client.post(url)

#     if project_id in leave_project_permission_map.get(username, []):
#         assert response.status_code == 302
#         assert not Membership.objects.filter(project_id=project_id, user__username=username).exists()
#     else:
#         if password:
#             assert response.status_code == 403
#         else:
#             assert response.status_code == 302



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from rdmo.core.utils import copy_model, join_url
from rdmo.domain.models import Attribute


class Condition(models.Model):

    RELATION_EQUAL = 'eq'
    RELATION_NOT_EQUAL = 'neq'
    RELATION_CONTAINS = 'contains'
    RELATION_GREATER_THAN = 'gt'
    RELATION_GREATER_THAN_EQUAL = 'gte'
    RELATION_LESSER_THAN = 'lt'
    RELATION_LESSER_THAN_EQUAL = 'lte'
    RELATION_EMPTY = 'empty'
    RELATION_NOT_EMPTY = 'notempty'
    RELATION_CHOICES = (
        (RELATION_EQUAL, 'is equal to (==)'),
        (RELATION_NOT_EQUAL, 'is not equal to (!=)'),
        (RELATION_CONTAINS, 'contains'),
        (RELATION_GREATER_THAN, 'is greater than (>)'),
        (RELATION_GREATER_THAN_EQUAL, 'is greater than or equal (>=)'),
        (RELATION_LESSER_THAN, 'is lesser than (<)'),
        (RELATION_LESSER_THAN_EQUAL, 'is lesser than or equal (<=)'),
        (RELATION_EMPTY, 'is empty'),
        (RELATION_NOT_EMPTY, 'is not empty'),
    )

    uri = models.URLField(
        max_length=640, blank=True,
        verbose_name=_('URI'),
        help_text=_('The Uniform Resource Identifier of this condition (auto-generated).')
    )
    uri_prefix = models.URLField(
        max_length=256,
        verbose_name=_('URI Prefix'),
        help_text=_('The prefix for the URI of this condition.')
    )
    key = models.SlugField(
        max_length=128, blank=True,
        verbose_name=_('Key'),
        help_text=_('The internal identifier of this condition.')
    )
    comment = models.TextField(
        blank=True,
        verbose_name=_('Comment'),
        help_text=_('Additional internal information about this condition.')
    )
    locked = models.BooleanField(
        default=False,
        verbose_name=_('Locked'),
        help_text=_('Designates whether this condition can be changed.')
    )
    source = models.ForeignKey(
        Attribute, db_constraint=False, blank=True, null=True, on_delete=models.SET_NULL, related_name='conditions',
        verbose_name=_('Source'),
        help_text=_('The attribute of the value for this condition.')
    )
    relation = models.CharField(
        max_length=8, choices=RELATION_CHOICES,
        verbose_name=_('Relation'),
        help_text=_('The relation this condition is using.')
    )
    target_text = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Target (Text)'),
        help_text=_('If using a regular value, the text value this condition is checking against (for boolean values use 1 and 0).')
    )
    target_option = models.ForeignKey(
        'options.Option', db_constraint=False, blank=True, null=True, on_delete=models.SET_NULL, related_name='conditions',
        verbose_name=_('Target (Option)'),
        help_text=_('If using a value pointing to an option, the option this condition is checking against.')
    )

    class Meta:
        ordering = ('uri', )
        verbose_name = _('Condition')
        verbose_name_plural = _('Conditions')

    def __str__(self):
        return self.key

    def save(self, *args, **kwargs):
        self.uri = self.build_uri(self.uri_prefix, self.key)
        super().save(*args, **kwargs)

    def copy(self, uri_prefix, key):
        condition = copy_model(self, uri_prefix=uri_prefix, key=key, source=self.source, target_option=self.target_option)

        return condition

    @property
    def source_label(self):
        return self.source.uri

    @property
    def relation_label(self):
        return self.get_relation_display()

    @property
    def target_label(self):
        if self.target_option:
            return self.target_option.label
        else:
            return self.target_text

    @property
    def is_locked(self):
        return self.locked

    def resolve(self, values, set_prefix=None, set_index=None, question=None):
        source_values = filter(lambda value: value.attribute == self.source, values)

        if set_prefix is not None:
            source_values = filter(lambda value: value.set_prefix == set_prefix, source_values)

        if set_index is not None:
            if question and not question.questionset.is_collection:
                # use the value without set_index since the
                # conditions refers to a non non set question
                pass
            else:
                source_values = filter(lambda value: value.set_index == int(set_index), source_values)

        source_values = list(source_values)
        if not source_values:
            if set_prefix:
                # try one level higher
                rpartition = set_prefix.rpartition('|')
                set_prefix, set_index = rpartition[0], int(rpartition[2])
                return self.resolve(values, set_prefix, set_index, question)

        if self.relation == self.RELATION_EQUAL:
            return self._resolve_equal(source_values)

        elif self.relation == self.RELATION_NOT_EQUAL:
            return not self._resolve_equal(source_values)

        elif self.relation == self.RELATION_CONTAINS:
            return self._resolve_contains(source_values)

        elif self.relation == self.RELATION_GREATER_THAN:
            return self._resolve_greater_than(source_values)

        elif self.relation == self.RELATION_GREATER_THAN_EQUAL:
            return self._resolve_greater_than_equal(source_values)

        elif self.relation == self.RELATION_LESSER_THAN:
            return self._resolve_lesser_than(source_values)

        elif self.relation == self.RELATION_LESSER_THAN_EQUAL:
            return self._resolve_lesser_than_equal(source_values)

        elif self.relation == self.RELATION_EMPTY:
            return not self._resolve_not_empty(source_values)

        elif self.relation == self.RELATION_NOT_EMPTY:
            return self._resolve_not_empty(source_values)

        else:
            return False

    def _resolve_equal(self, values):
        results = []

        for value in values:
            if self.target_option:
                results.append(value.option == self.target_option)
            else:
                results.append(value.text == self.target_text)

        return True in results

    def _resolve_contains(self, values):
        results = []

        for value in values:
            results.append(self.target_text in value.text)

        return True in results

    def _resolve_greater_than(self, values):

        for value in values:
            try:
                if float(value.text) > float(self.target_text):
                    return True
            except ValueError:
                pass

        return False

    def _resolve_greater_than_equal(self, values):

        for value in values:
            try:
                if float(value.text) >= float(self.target_text):
                    return True
            except ValueError:
                pass

        return False

    def _resolve_lesser_than(self, values):

        for value in values:
            try:
                if float(value.text) < float(self.target_text):
                    return True
            except ValueError:
                pass

        return False

    def _resolve_lesser_than_equal(self, values):

        for value in values:
            try:
                if float(value.text) <= float(self.target_text):
                    return True
            except ValueError:
                pass

        return False

    def _resolve_not_empty(self, values):

        for value in values:
            if bool(value.text) or bool(value.option):
                return True

        return False

    @classmethod
    def build_uri(cls, uri_prefix, key):
        assert key
        return join_url(uri_prefix or settings.DEFAULT_URI_PREFIX, '/conditions/', key)



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Attribute
from ..serializers.v1 import AttributeSerializer
from ..validators import AttributeParentValidator


def test_create(db):
    AttributeParentValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'parent': Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
    })


def test_update(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    AttributeParentValidator(attribute)({
        'uri_prefix': attribute.uri_prefix,
        'key': attribute.key,
        'parent': attribute.parent
    })


def test_update_error(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    with pytest.raises(ValidationError):
        AttributeParentValidator(attribute)({
            'uri_prefix': attribute.uri_prefix,
            'key': attribute.key,
            'parent': attribute  # set self as parent
        })


def test_serializer_create(db):
    class MockedView(object):
        action = 'create'

    validator = AttributeParentValidator()
    validator.set_context(AttributeSerializer())
    validator.serializer.context['view'] = MockedView()

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'parent': Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
    })


def test_serializer_update(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    validator = AttributeParentValidator()
    validator.set_context(AttributeSerializer(instance=attribute))

    validator({
        'uri_prefix': attribute.uri_prefix,
        'key': attribute.key,
        'parent':  attribute.parent
    })


def test_serializer_update_error(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    validator = AttributeParentValidator()
    validator.set_context(AttributeSerializer(instance=attribute))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': attribute.uri_prefix,
            'key': attribute.key,
            'parent':  attribute  # set self as parent
        })


def test_serializer_copy(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')

    class MockedView(object):
        action = 'copy'

        def get_object(self):
            return attribute

    validator = AttributeParentValidator()
    validator.set_context(AttributeSerializer())
    validator.serializer.context['view'] = MockedView()

    validator({
        'uri_prefix': attribute.uri_prefix,
        'key': attribute.key,
        'parent':  attribute.parent
    })


def test_serializer_copy_error(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')

    class MockedView(object):
        action = 'copy'

        def get_object(self):
            return attribute

    validator = AttributeParentValidator()
    validator.set_context(AttributeSerializer())
    validator.serializer.context['view'] = MockedView()

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': attribute.uri_prefix,
            'key': attribute.key,
            'parent':  attribute  # set self as parent
        })



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations

from rdmo.core.utils import get_uri_prefix


def run_data_migration(apps, schema_editor):
    Section = apps.get_model('questions', 'Section')
    Subsection = apps.get_model('questions', 'Subsection')
    QuestionSet = apps.get_model('questions', 'QuestionSet')
    Question = apps.get_model('questions', 'Question')

    for questionset in QuestionSet.objects.all():
        questionset.section = questionset.subsection.section
        questionset.title_en = questionset.subsection.title_en
        questionset.title_de = questionset.subsection.title_de
        questionset.order = questionset.order + 10 * questionset.subsection.order

        questionset.key = questionset.subsection.key + '-' + questionset.key
        questionset.path = '%s/%s/%s' % (
            questionset.section.catalog.key,
            questionset.section.key,
            questionset.key
        )
        questionset.uri = get_uri_prefix(questionset) + '/questions/' + questionset.path
        questionset.save()

        for question in questionset.questions.all():
            question.path = '%s/%s/%s/%s' % (
                question.questionset.section.catalog.key,
                question.questionset.section.key,
                question.questionset.key,
                question.key
            )
            question.uri = get_uri_prefix(question) + '/questions/' + question.path
            question.save()


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0034_move_questionset_to_section'),
    ]

    operations = [
        migrations.RunPython(run_data_migration),
    ]



import logging

from django.core.management.base import BaseCommand, CommandError
from django.utils.translation import gettext_lazy as _

from rdmo.core.xml import flat_xml_to_elements, read_xml_file
from rdmo.management.imports import import_elements

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('xmlfile', action='store', default=False, help='RDMO XML export file')

    def handle(self, *args, **options):
        root = read_xml_file(options['xmlfile'])
        if root is None:
            raise CommandError(_('The content of the xml file does not consist of well formed data or markup.'))
        elif root.tag != 'rdmo':
            raise CommandError(_('This XML does not contain RDMO content.'))
        else:
            elements = flat_xml_to_elements(root)
            save = {element.get('uri'): True for element in elements}
            import_elements(elements, save=save)



import pytest
from django.contrib.auth import get_user_model
from django.urls import reverse

from ..models import Invite, Membership, Project

membership_roles = ('owner', 'manager', 'author', 'guest')


@pytest.fixture()
def use_project_invite_timeout(settings):
    settings.PROJECT_INVITE_TIMEOUT = 0


@pytest.mark.parametrize('membership_role', membership_roles)
def test_project_join(db, client, membership_role):
    client.login(username='user', password='user')

    project = Project.objects.get(id=1)
    user = get_user_model().objects.get(username='user')
    invite = Invite(project=project, user=user, role=membership_role)
    invite.make_token()
    invite.save()

    url = reverse('project_join', args=[invite.token])
    response = client.get(url)

    assert response.status_code == 302
    assert Membership.objects.get(project=project, user=user, role=membership_role)


@pytest.mark.parametrize('membership_role', membership_roles)
def test_project_join_mail(db, client, membership_role):
    client.login(username='user', password='user')

    project = Project.objects.get(id=1)
    user = get_user_model().objects.get(username='user')
    invite = Invite(project=project, user=None, role=membership_role)
    invite.make_token()
    invite.save()

    url = reverse('project_join', args=[invite.token])
    response = client.get(url)

    assert response.status_code == 302
    assert Membership.objects.get(project=project, user=user, role=membership_role)


@pytest.mark.parametrize('membership_role', membership_roles)
def test_project_join_mail_existing_user(db, client, membership_role):
    client.login(username='author', password='author')

    project = Project.objects.get(id=1)
    user = get_user_model().objects.get(username='author')
    invite = Invite(project=project, user=None, role=membership_role)
    invite.make_token()
    invite.save()

    url = reverse('project_join', args=[invite.token])
    response = client.get(url)

    membership = Membership.objects.get(project=project, user=user)

    assert response.status_code == 302
    assert membership.role == 'author'


@pytest.mark.parametrize('membership_role', membership_roles)
def test_project_join_error(db, client, membership_role):
    client.login(username='user', password='user')

    project = Project.objects.get(id=1)
    user = get_user_model().objects.get(username='user')
    invite = Invite(project=project, user=user, role=membership_role)
    invite.make_token()
    invite.save()

    url = reverse('project_join', args=['wrong'])
    response = client.get(url)

    assert response.status_code == 200
    assert b'is not valid' in response.content
    assert not Membership.objects.filter(project=project, user=user, role=membership_role).exists()


@pytest.mark.parametrize('membership_role', membership_roles)
def test_project_join_timeout_error(db, client, membership_role, use_project_invite_timeout):
    client.login(username='user', password='user')

    project = Project.objects.get(id=1)
    user = get_user_model().objects.get(username='user')
    invite = Invite(project=project, user=user, role=membership_role)
    invite.make_token()
    invite.save()

    url = reverse('project_join', args=[invite.token])
    response = client.get(url)

    assert response.status_code == 200
    assert b'expired' in response.content
    assert not Membership.objects.filter(project=project, user=user, role=membership_role).exists()


@pytest.mark.parametrize('membership_role', membership_roles)
def test_project_join_user_error(db, client, membership_role):
    client.login(username='user', password='user')

    project = Project.objects.get(id=1)
    user = get_user_model().objects.get(username='user')
    invite = Invite(project=project, user=get_user_model().objects.get(username='guest'), role=membership_role)
    invite.make_token()
    invite.save()

    url = reverse('project_join', args=[invite.token])
    response = client.get(url)

    assert response.status_code == 200
    assert b'guest' in response.content
    assert not Membership.objects.filter(project=project, user=user, role=membership_role).exists()



from rest_framework import serializers

from rdmo.projects.models import Project
from rdmo.questions.models import Catalog, QuestionSet, Section


class QuestionSetSerializer(serializers.ModelSerializer):

    class Meta:
        model = QuestionSet
        fields = (
            'id',
            'title',
            'has_conditions'
        )


class SectionSerializer(serializers.ModelSerializer):

    questionsets = QuestionSetSerializer(many=True, read_only=True)

    class Meta:
        model = Section
        fields = (
            'id',
            'title',
            'questionsets'
        )


class CatalogSerializer(serializers.ModelSerializer):

    sections = SectionSerializer(many=True, read_only=True)

    class Meta:
        model = Catalog
        fields = (
            'id',
            'title',
            'sections'
        )


class ProjectOverviewSerializer(serializers.ModelSerializer):

    catalog = CatalogSerializer()
    read_only = serializers.SerializerMethodField()

    class Meta:
        model = Project
        fields = (
            'id',
            'title',
            'catalog',
            'read_only'
        )

    def get_read_only(self, obj):
        request = self.context.get('request')

        if request:
            return not (request.user.has_perm('projects.add_value_object', obj) and
                        request.user.has_perm('projects.change_value_object', obj) and
                        request.user.has_perm('projects.delete_value_object', obj))
        else:
            return True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0002_detail_key_type_field_length_increased'),
    ]

    operations = [
        migrations.RenameField(
            model_name='detailkey',
            old_name='hint',
            new_name='help_text',
        ),
        migrations.AlterField(
            model_name='detailkey',
            name='options',
            field=jsonfield.fields.JSONField(blank=True, null=True, help_text='Enter valid JSON of the form [[key, label], [key, label], ...]'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-09-23 14:24
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0003_meta'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='snapshot',
            options={'ordering': ('project', '-created'), 'verbose_name': 'Snapshot', 'verbose_name_plural': 'Snapshots'},
        ),
        migrations.RemoveField(
            model_name='project',
            name='current_snapshot',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-16 10:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0018_data_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='value',
            name='value_type',
            field=models.CharField(choices=[(b'text', 'Text'), (b'url', 'URL'), (b'integer', 'Integer'), (b'float', 'Float'), (b'boolean', 'Boolean'), (b'datetime', 'Datetime'), (b'option', 'Option')], help_text='Type of this value.', max_length=8, verbose_name='Value type'),
        ),
    ]



import rules
from django.contrib.sites.models import Site

from .utils import is_site_manager as is_site_manager_util


@rules.predicate
def is_site_manager(manager, user):
    if is_site_manager_util(manager):
        current_site = Site.objects.get_current()
        return current_site in user.role.member.all()
    else:
        return False


rules.add_perm('auth.view_user_object', is_site_manager)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-25 09:33
from __future__ import unicode_literals

from django.db import migrations


def migrate_options(apps, schema_editor):

    Option = apps.get_model('options', 'Option')
    OptionSet = apps.get_model('options', 'OptionSet')
    OldOption = apps.get_model('domain', 'Option')

    attribute_ids = set(OldOption.objects.values_list('attribute', flat=True))
    for attribute_id in attribute_ids:

        # create an OptionSet for this attribute_id
        optionset = OptionSet.objects.create(
            title=attribute_id,
            order=0,
        )

        # attribute = Attribute.objects.filter(pk=attribute_id)
        # attribute.optionsets.add(optionset)
        # attribute.save()

        old_options = OldOption.objects.filter(attribute=attribute_id)

        # add the optionset to the attribute
        attribute = old_options.first().attribute
        attribute.optionsets.add(optionset)
        attribute.save()

        # loop over old options for this attribute and create new options
        for option in old_options:

            Option.objects.create(
                pk=option.pk,
                optionset=optionset,
                order=option.order,
                title=option.pk,
                text_en=option.text_en,
                text_de=option.text_de,
                additional_input=option.additional_input
            )


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0002_meta'),
        ('domain', '0021_options'),
        ('projects', '0009_options'),
    ]

    operations = [
        migrations.RunPython(migrate_options),
    ]



from rdmo.questions.widgets import Widget


class TextWidget(Widget):
    template_name = 'projects/project_questions_form_group_text.html'
    widget_class = 'text'


class TextareaWidget(Widget):
    template_name = 'projects/project_questions_form_group_textarea.html'
    widget_class = 'text'


class YesnoWidget(Widget):
    template_name = 'projects/project_questions_form_group_yesno.html'
    widget_class = 'yesno'


class RadioWidget(Widget):
    template_name = 'projects/project_questions_form_group_radio.html'
    widget_class = 'radio'


class SelectWidget(Widget):
    template_name = 'projects/project_questions_form_group_select.html'
    widget_class = 'select'


class AutocompleteWidget(Widget):
    template_name = 'projects/project_questions_form_group_autocomplete.html'
    widget_class = 'autocomplete'


class DateWidget(Widget):
    template_name = 'projects/project_questions_form_group_date.html'
    widget_class = 'date'


class RangeWidget(Widget):
    template_name = 'projects/project_questions_form_group_range.html'
    widget_class = 'range'


class CheckboxWidget(Widget):
    template_name = 'projects/project_questions_form_group_checkbox.html'
    widget_class = 'checkbox'


class FileWidget(Widget):
    template_name = 'projects/project_questions_form_group_file.html'
    widget_class = 'file'



# Generated by Django 2.2.18 on 2021-04-20 13:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0058_question_default_option'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='default_external_id',
            field=models.CharField(blank=True, help_text='The default external id for this question. To be used with dynamic optionsets.', max_length=256, verbose_name='Default external id'),
        ),
    ]



import pytest
from django.urls import reverse

from ..models import Issue

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('api', 'api'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

view_issue_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'author': [1, 3, 5],
    'guest': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

urlnames = {
    'list': 'v1-projects:issue-list',
    'detail': 'v1-projects:issue-detail'
}

projects = [1, 2, 3, 4, 5]
issues = [1, 2, 3, 4]

site_id = 1
project_id = 1
project_count = 4

issue_status = ('open', 'in_progress', 'closed')


@pytest.mark.parametrize('username,password', users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.get(url)

    if password:
        assert response.status_code == 200
        assert isinstance(response.json(), list)

        if username == 'user':
            assert sorted([item['id'] for item in response.json()]) == []
        else:
            values_list = Issue.objects.filter(project__in=view_issue_permission_map.get(username, [])) \
                                       .order_by('id').values_list('id', flat=True)
            assert sorted([item['id'] for item in response.json()]) == list(values_list)
    else:
        assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('issue_id', issues)
def test_detail(db, client, username, password, issue_id):
    client.login(username=username, password=password)
    issue = Issue.objects.get(id=issue_id)

    url = reverse(urlnames['detail'], args=[issue_id])
    response = client.get(url)

    if issue.project.id in view_issue_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), dict)
        assert response.json().get('id') == issue_id
    elif password:
        assert response.status_code == 404
    else:
        assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
def test_create(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'])
    response = client.post(url)

    if password:
        assert response.status_code == 405
    else:
        assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('issue_id', issues)
def test_update(db, client, username, password, issue_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[issue_id])
    data = {}
    response = client.put(url, data, content_type='application/json')

    if password:
        assert response.status_code == 405
    else:
        assert response.status_code == 401


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('issue_id', issues)
def test_delete(db, client, username, password, issue_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['detail'], args=[issue_id])
    response = client.delete(url)

    if password:
        assert response.status_code == 405
    else:
        assert response.status_code == 401



import logging
import tempfile
import time
from os.path import join as pj
from random import randint

from django.core.exceptions import ObjectDoesNotExist, ValidationError

from rdmo.core.utils import get_languages

logger = logging.getLogger(__name__)


def handle_uploaded_file(filedata):
    tempfilename = generate_tempfile_name()
    with open(tempfilename, 'wb+') as destination:
        for chunk in filedata.chunks():
            destination.write(chunk)
    return tempfilename


def generate_tempfile_name():
    t = int(round(time.time() * 1000))
    r = randint(10000, 99999)
    fn = pj(tempfile.gettempdir(), 'upload_' + str(t) + '_' + str(r) + '.xml')
    return fn


def set_common_fields(instance, element):
    instance.uri_prefix = element.get('uri_prefix') or ''
    instance.key = element.get('key') or ''
    instance.comment = element.get('comment') or ''

    # these are temporary fields for the import only
    # uri is set automatically when saving the instance
    instance.uri = element.get('uri')
    instance.object_name = instance._meta.object_name
    instance.missing = {}
    instance.errors = []


def set_lang_field(instance, field_name, element):
    for lang_code, lang_string, lang_field in get_languages():
        field = element.get('%s_%s' % (field_name, lang_code))
        if field:
            setattr(instance, '%s_%s' % (field_name, lang_field), field)
        else:
            setattr(instance, '%s_%s' % (field_name, lang_field), '')


def get_foreign_field(instance, foreign_uri, foreign_model):
    if foreign_uri:
        try:
            return foreign_model.objects.get(uri=foreign_uri)
        except foreign_model.DoesNotExist:
            logger.info('{foreign_model} {foreign_uri} for {instance_model} {instance_uri} does not exist.'.format(
                foreign_model=foreign_model._meta.object_name,
                foreign_uri=foreign_uri,
                instance_model=instance._meta.object_name,
                instance_uri=instance.uri
            ))
            instance.missing[foreign_uri] = {
                'foreign_model': foreign_model._meta.verbose_name,
                'foreign_uri': foreign_uri
            }

    # return None by default
    return None


def get_m2m_instances(instance, foreign_uris, foreign_model):
    foreign_instances = []

    if foreign_uris:
        for foreign_uri in foreign_uris:
            try:
                foreign_instance = foreign_model.objects.get(uri=foreign_uri)
                foreign_instances.append(foreign_instance)
            except foreign_model.DoesNotExist:
                logger.info('{foreign_model} {foreign_uri} for imported {instance_model} {instance_uri} does not exist.'.format(
                    foreign_model=foreign_model._meta.object_name,
                    foreign_uri=foreign_uri,
                    instance_model=instance._meta.object_name,
                    instance_uri=instance.uri
                ))
                instance.missing[foreign_uri] = {
                    'foreign_model': foreign_model._meta.verbose_name,
                    'foreign_uri': foreign_uri
                }

    return foreign_instances


def validate_instance(instance, *validators):
    exception_message = None
    try:
        for validator in validators:
            data = vars(instance)

            # add foreign keys to parent fields to data
            for parent_field in getattr(instance, 'parent_fields', []):
                data[parent_field] = getattr(instance, parent_field, None)

            # run the validator
            validator(instance if instance.id else None)(data)

    except ValidationError as e:
        try:
            exception_message = '; '.join(['{}: {}'.format(key, ', '.join(messages)) for key, messages in e.message_dict.items()])
        except AttributeError:
            exception_message = ''.join(e.messages)
    except ObjectDoesNotExist as e:
        exception_message = e
    else:
        return True
    finally:
        if exception_message is not None:
            message = '{instance_model} {instance_uri} cannot be imported ({exception}).'.format(
                instance_model=instance._meta.object_name,
                instance_uri=instance.uri,
                exception=exception_message
            )
            logger.info(message)
            instance.errors.append(message)


def fetch_parents(model, instances):
    parents = list(model.objects.values_list('uri', flat=True))
    return set(sorted(parents))



import pytest
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Catalog, Section
from ..serializers.v1 import SectionSerializer
from ..validators import SectionLockedValidator


def test_create(db):
    SectionLockedValidator()({
        'catalog': Catalog.objects.first(),
        'locked': False
    })


def test_create_locked(db):
    SectionLockedValidator()({
        'catalog': Catalog.objects.first(),
        'locked': True
    })


def test_update(db):
    section = Section.objects.first()

    SectionLockedValidator(section)({
        'catalog': section.catalog,
        'locked': False
    })


def test_update_error(db):
    section = Section.objects.first()
    section.locked = True
    section.save()

    with pytest.raises(ValidationError):
        SectionLockedValidator(section)({
            'catalog': section.catalog,
            'locked': True
        })


def test_update_parent_error(db):
    section = Section.objects.first()
    section.catalog.locked = True
    section.catalog.save()

    with pytest.raises(ValidationError):
        SectionLockedValidator(section)({
            'catalog': section.catalog,
            'locked': False
        })


def test_update_lock(db):
    section = Section.objects.first()

    SectionLockedValidator(section)({
        'catalog': section.catalog,
        'locked': True
    })


def test_update_unlock(db):
    section = Section.objects.first()
    section.locked = True
    section.save()

    SectionLockedValidator(section)({
        'catalog': section.catalog,
        'locked': False
    })


def test_serializer_create(db):
    validator = SectionLockedValidator()
    validator.set_context(SectionSerializer())

    validator({
        'catalog': Catalog.objects.first(),
        'locked': False
    })


def test_serializer_create_locked(db):
    validator = SectionLockedValidator()
    validator.set_context(SectionSerializer())

    validator({
        'catalog': Catalog.objects.first(),
        'locked': True
    })


def test_serializer_update(db):
    section = Section.objects.first()

    validator = SectionLockedValidator()
    validator.set_context(SectionSerializer(instance=section))

    validator({
        'catalog': section.catalog,
        'locked': False
    })


def test_serializer_update_error(db):
    section = Section.objects.first()
    section.locked = True
    section.save()

    validator = SectionLockedValidator()
    validator.set_context(SectionSerializer(instance=section))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'catalog': section.catalog,
            'locked': True
        })


def test_serializer_update_parent_error(db):
    section = Section.objects.first()
    section.catalog.locked = True
    section.catalog.save()

    validator = SectionLockedValidator()
    validator.set_context(SectionSerializer(instance=section))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'catalog': section.catalog,
            'locked': True
        })


def test_serializer_update_lock(db):
    section = Section.objects.first()

    validator = SectionLockedValidator()
    validator.set_context(SectionSerializer(instance=section))

    validator({
        'catalog': section.catalog,
        'locked': True
    })


def test_serializer_update_unlock(db):
    section = Section.objects.first()
    section.locked = True
    section.save()

    validator = SectionLockedValidator()
    validator.set_context(SectionSerializer(instance=section))

    validator({
        'catalog': section.catalog,
        'locked': False
    })



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-25 11:32
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0020_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='attribute',
            name='optionsets',
            field=models.ManyToManyField(blank=True, to='options.OptionSet'),
        ),
    ]



# Generated by Django 2.2.18 on 2021-03-29 13:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0056_question_is_optional'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='default_text_lang1',
            field=models.TextField(blank=True, help_text='The default text value for this question in the primary language.', null=True, verbose_name='Default text value (primary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='default_text_lang2',
            field=models.TextField(blank=True, help_text='The default text value for this question in the secondary language.', null=True, verbose_name='Default text value (secondary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='default_text_lang3',
            field=models.TextField(blank=True, help_text='The default text value for this question in the tertiary language.', null=True, verbose_name='Default text value (tertiary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='default_text_lang4',
            field=models.TextField(blank=True, help_text='The default text value for this question in the quaternary language.', null=True, verbose_name='Default text value (quaternary)'),
        ),
        migrations.AddField(
            model_name='question',
            name='default_text_lang5',
            field=models.TextField(blank=True, help_text='The default text value for this question in the quinary language.', null=True, verbose_name='Default text value (quinary)'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-07-18 11:25
from __future__ import unicode_literals

import rdmo.core.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('domain', '0008_meta'),
    ]

    operations = [
        migrations.CreateModel(
            name='Task',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_period', models.DurationField()),
                ('title_en', models.CharField(max_length=256)),
                ('title_de', models.CharField(max_length=256)),
                ('text_en', models.CharField(max_length=256)),
                ('text_de', models.CharField(max_length=256)),
                ('attribute', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Attribute')),
            ],
            options={
                'ordering': ('attribute',),
                'verbose_name': 'Task',
                'verbose_name_plural': 'Tasks',
            },
            bases=(rdmo.core.models.TranslationMixin, models.Model),
        ),
    ]



from pathlib import Path

import pytest
from django.conf import settings
from django.urls import reverse

from rdmo.core.constants import VALUE_TYPE_FILE

from ..models import Project, Snapshot

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('api', 'api'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

view_snapshot_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'author': [1, 3, 5],
    'guest': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

add_snapshot_permission_map = change_snapshot_permission_map = delete_snapshot_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

urlnames = {
    'list': 'v1-projects:project-snapshot-list',
    'detail': 'v1-projects:project-snapshot-detail'
}

projects = [1, 2, 3, 4, 5]
snapshots = [1, 3, 7, 4, 5, 6]


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_list(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse(urlnames['list'], args=[project_id])
    response = client.get(url)

    if project_id in view_snapshot_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), list)

        if username == 'user':
            assert sorted([item['id'] for item in response.json()]) == []
        else:
            values_list = Snapshot.objects.filter(project_id=project_id) \
                                          .order_by('id').values_list('id', flat=True)
            assert sorted([item['id'] for item in response.json()]) == list(values_list)

    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('snapshot_id', snapshots)
def test_detail(db, client, username, password, project_id, snapshot_id):
    client.login(username=username, password=password)
    snapshot = Snapshot.objects.filter(project_id=project_id, id=snapshot_id).filter()

    url = reverse(urlnames['detail'], args=[project_id, snapshot_id])
    response = client.get(url)

    if snapshot and project_id in view_snapshot_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), dict)
        assert response.json().get('id') == snapshot_id
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_create(db, client, files, username, password, project_id):
    client.login(username=username, password=password)
    project = Project.objects.get(id=project_id)

    snapshot_count = project.snapshots.count()
    values_count = project.values.count()
    current_values_count = project.values.filter(snapshot=None).count()

    url = reverse(urlnames['list'], args=[project_id])
    data = {
        'title': 'A new snapshot',
        'description': 'Some description'
    }
    response = client.post(url, data)

    if project_id in add_snapshot_permission_map.get(username, []):
        assert response.status_code == 201
        assert isinstance(response.json(), dict)
        assert response.json().get('id') in project.snapshots.values_list('id', flat=True)
        assert project.snapshots.count() == snapshot_count + 1
        assert project.values.count() == values_count + current_values_count

        for file_value in project.values.filter(value_type=VALUE_TYPE_FILE):
            assert Path(settings.MEDIA_ROOT).joinpath(file_value.file.name).exists()
    else:
        if project_id in view_snapshot_permission_map.get(username, []):
            assert response.status_code == 403
        else:
            assert response.status_code == 404

        assert project.snapshots.count() == snapshot_count
        assert project.values.count() == values_count


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('snapshot_id', snapshots)
def test_update(db, client, username, password, project_id, snapshot_id):
    client.login(username=username, password=password)
    project = Project.objects.get(id=project_id)
    snapshot = Snapshot.objects.filter(project_id=project_id, id=snapshot_id).first()

    snapshot_count = project.snapshots.count()
    values_count = project.values.count()
    values_files = [value.file.name for value in project.values.filter(value_type=VALUE_TYPE_FILE)]

    url = reverse(urlnames['detail'], args=[project_id, snapshot_id])
    data = {
        'title': 'A new title',
        'description': 'A new description'
    }
    response = client.put(url, data, content_type='application/json')

    if snapshot and project_id in change_snapshot_permission_map.get(username, []):
        assert response.status_code == 200
        assert isinstance(response.json(), dict)
        assert response.json().get('id') in project.snapshots.values_list('id', flat=True)
    elif snapshot and project_id in view_snapshot_permission_map.get(username, []):
        assert response.status_code == 403
    else:
        assert response.status_code == 404

    assert project.snapshots.count() == snapshot_count
    assert project.values.count() == values_count
    for file_value in values_files:
        assert Path(settings.MEDIA_ROOT).joinpath(file_value).exists()


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('snapshot_id', snapshots)
def test_delete(db, client, username, password, project_id, snapshot_id):
    client.login(username=username, password=password)
    project = Project.objects.get(id=project_id)

    snapshot_count = project.snapshots.count()
    values_count = project.values.count()
    values_files = [value.file.name for value in project.values.filter(value_type=VALUE_TYPE_FILE)]

    url = reverse(urlnames['detail'], args=[project_id, snapshot_id])
    response = client.delete(url)

    if project_id in view_snapshot_permission_map.get(username, []):
        assert response.status_code == 405
    else:
        assert response.status_code == 404

    assert project.snapshots.count() == snapshot_count
    assert project.values.count() == values_count
    for file_value in values_files:
        assert Path(settings.MEDIA_ROOT).joinpath(file_value).exists()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-25 14:23
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0014_remove_owner'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='membership',
            options={'ordering': ('project__title',), 'permissions': (('view_membership', 'Can view membership'),), 'verbose_name': 'Membership', 'verbose_name_plural': 'Memberships'},
        ),
        migrations.AlterModelOptions(
            name='project',
            options={'ordering': ('title',), 'permissions': (('view_project', 'Can view project'),), 'verbose_name': 'Project', 'verbose_name_plural': 'Projects'},
        ),
        migrations.AlterModelOptions(
            name='snapshot',
            options={'ordering': ('project', '-created'), 'permissions': (('view_snapshot', 'Can view snapshot'),), 'verbose_name': 'Snapshot', 'verbose_name_plural': 'Snapshots'},
        ),
        migrations.AlterModelOptions(
            name='value',
            options={'permissions': (('view_value', 'Can view value'),), 'verbose_name': 'Value', 'verbose_name_plural': 'Values'},
        ),
    ]



import subprocess

from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def handle(self, *args, **options):

        print('>>> python manage.py collectstatic --noinput --clear')
        print()
        call_command('collectstatic', '--noinput', '--clear')
        print()

        print('>>> touch config/wsgi.py')
        print()
        subprocess.call(['touch', 'config/wsgi.py'])
        print()



# Generated by Django 2.2.13 on 2020-09-10 15:27

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0037_project_tasks'),
    ]

    operations = [
        migrations.CreateModel(
            name='Integration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('provider_key', models.TextField(help_text='The key of the provider for this integration.', verbose_name='Provider key')),
                ('project', models.ForeignKey(help_text='The project for this integration.', on_delete=django.db.models.deletion.CASCADE, related_name='integrations', to='projects.Project', verbose_name='Project')),
            ],
            options={
                'verbose_name': 'Integration',
                'verbose_name_plural': 'Integrations',
                'ordering': ('project__title',),
            },
        ),
        migrations.CreateModel(
            name='IntegrationOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.SlugField(help_text='The key for this integration option.', max_length=128, verbose_name='Key')),
                ('value', models.TextField(help_text='The value for this integration option.', verbose_name='Value')),
                ('integration', models.ForeignKey(help_text='The integration for this integration option.', on_delete=django.db.models.deletion.CASCADE, related_name='options', to='projects.Integration', verbose_name='Integration')),
            ],
            options={
                'verbose_name': 'Integration option',
                'verbose_name_plural': 'Integration options',
                'ordering': ('integration__project__title',),
            },
        ),
    ]



from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from django.db.models import Prefetch
from django.http import Http404, HttpResponseRedirect
from django.utils.translation import gettext_lazy as _
from django_filters.rest_framework import DjangoFilterBackend

from rest_framework import serializers
from rest_framework.decorators import action
from rest_framework.exceptions import NotFound
from rest_framework.mixins import (CreateModelMixin, ListModelMixin,
                                   RetrieveModelMixin, UpdateModelMixin)
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.reverse import reverse
from rest_framework.viewsets import (GenericViewSet, ModelViewSet,
                                     ReadOnlyModelViewSet)
from rest_framework_extensions.mixins import NestedViewSetMixin

from rdmo.conditions.models import Condition
from rdmo.core.permissions import HasModelPermission, HasObjectPermission
from rdmo.core.utils import human2bytes, return_file_response
from rdmo.options.models import OptionSet
from rdmo.questions.models import Catalog, Question, QuestionSet

from .filters import SnapshotFilterBackend, ValueFilterBackend
from .models import (Continuation, Integration, Issue, Membership, Project,
                     Snapshot, Value)
from .serializers.v1 import (IntegrationSerializer, IssueSerializer,
                             MembershipSerializer,
                             ProjectIntegrationSerializer,
                             ProjectIssueSerializer,
                             ProjectMembershipSerializer,
                             ProjectMembershipUpdateSerializer,
                             ProjectSerializer, ProjectSnapshotSerializer,
                             ProjectValueSerializer, SnapshotSerializer,
                             ValueSerializer)
from .serializers.v1.overview import ProjectOverviewSerializer
from .serializers.v1.questionset import QuestionSetSerializer
from .utils import check_conditions


class ProjectViewSet(ModelViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = ProjectSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'title',
        'user',
        'user__username',
        'catalog',
        'catalog__uri',
        'catalog__key',
    )

    def get_queryset(self):
        return Project.objects.filter_user(self.request.user)

    @action(detail=True, permission_classes=(IsAuthenticated, ))
    def overview(self, request, pk=None):
        project = self.get_object()
        project.catalog = Catalog.objects.prefetch_related(
            'sections',
            Prefetch('sections__questionsets', queryset=QuestionSet.objects.filter(questionset=None).prefetch_related(
                'conditions',
                'questions'
            ))
        ).get(id=project.catalog_id)
        serializer = ProjectOverviewSerializer(project, context={'request': request})
        return Response(serializer.data)

    @action(detail=True, permission_classes=(HasModelPermission | HasObjectPermission, ))
    def resolve(self, request, pk=None):
        snapshot_id = request.GET.get('snapshot')
        set_prefix = request.GET.get('set_prefix')
        set_index = request.GET.get('set_index')

        values = self.get_object().values.filter(snapshot_id=snapshot_id).select_related('attribute', 'option')

        questionset_id = request.GET.get('questionset')
        if questionset_id:
            try:
                questionset = QuestionSet.objects.get(id=questionset_id)
                conditions = questionset.conditions.select_related('source', 'target_option')
                if check_conditions(conditions, values, set_prefix, set_index):
                    return Response({'result': True})
            except QuestionSet.DoesNotExist:
                pass

        question_id = request.GET.get('question')
        if question_id:
            try:
                question = Question.objects.get(id=question_id)
                conditions = question.conditions.select_related('source', 'target_option')
                if check_conditions(conditions, values, set_prefix, set_index):
                    return Response({'result': True})
            except Question.DoesNotExist:
                pass

        optionset_id = request.GET.get('optionset')
        if optionset_id:
            try:
                optionset = OptionSet.objects.get(id=optionset_id)
                conditions = optionset.conditions.select_related('source', 'target_option')
                if check_conditions(conditions, values, set_prefix, set_index):
                    return Response({'result': True})
            except OptionSet.DoesNotExist:
                pass

        condition_id = request.GET.get('condition')
        if condition_id:
            try:
                condition = Condition.objects.select_related('source', 'target_option').get(id=condition_id)
                if check_conditions([condition], values, set_prefix, set_index):
                    return Response({'result': True})
            except Condition.DoesNotExist:
                pass

        return Response({'result': False})

    @action(detail=True, permission_classes=(HasModelPermission | HasObjectPermission, ))
    def options(self, request, pk=None):
        project = self.get_object()

        try:
            try:
                optionset_id = request.GET.get('optionset')
                optionset = OptionSet.objects.get(pk=optionset_id)
            except (ValueError, OptionSet.DoesNotExist):
                raise NotFound()

            # check if the optionset belongs to this catalog and if it has a provider
            if Question.objects.filter_by_catalog(project.catalog).filter(optionsets=optionset) and \
                    optionset.provider is not None:
                options = optionset.provider.get_options(project, search=request.GET.get('search'))
                return Response(options)

        except OptionSet.DoesNotExist:
            pass

        # if it didn't work return 404
        raise NotFound()

    @action(detail=True, permission_classes=(IsAuthenticated, ))
    def progress(self, request, pk=None):
        project = self.get_object()
        return Response(project.progress)

    def perform_create(self, serializer):
        project = serializer.save(site=get_current_site(self.request))

        # add current user as owner
        membership = Membership(project=project, user=self.request.user, role='owner')
        membership.save()


class ProjectNestedViewSetMixin(NestedViewSetMixin):

    def initial(self, request, *args, **kwargs):
        self.project = self.get_project_from_parent_viewset()
        super().initial(request, *args, **kwargs)

    def get_project_from_parent_viewset(self):
        try:
            return Project.objects.filter_user(self.request.user).get(pk=self.get_parents_query_dict().get('project'))
        except Project.DoesNotExist:
            raise Http404

    def get_list_permission_object(self):
        return self.project

    def get_detail_permission_object(self, obj):
        return self.project

    def perform_create(self, serializer):
        serializer.save(project=self.project)


class ProjectMembershipViewSet(ProjectNestedViewSetMixin, ModelViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )

    filter_backends = (DjangoFilterBackend, )
    filterset_fields = (
        'user',
        'user__username',
        'role'
    )

    def get_queryset(self):
        try:
            return Membership.objects.filter(project=self.project)
        except AttributeError:
            # this is needed for the swagger ui
            return Membership.objects.none()

    def get_serializer_class(self):
        if self.action == 'update':
            return ProjectMembershipUpdateSerializer
        else:
            return ProjectMembershipSerializer


class ProjectIntegrationViewSet(ProjectNestedViewSetMixin, ModelViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = ProjectIntegrationSerializer

    filter_backends = (DjangoFilterBackend, )
    filterset_fields = (
        'provider_key',
    )

    def get_queryset(self):
        try:
            return Integration.objects.filter(project=self.project)
        except AttributeError:
            # this is needed for the swagger ui
            return Integration.objects.none()


class ProjectIssueViewSet(ProjectNestedViewSetMixin, ListModelMixin, RetrieveModelMixin,
                          UpdateModelMixin, GenericViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = ProjectIssueSerializer

    filter_backends = (DjangoFilterBackend, )
    filterset_fields = (
        'task',
        'task__uri',
        'status'
    )

    def get_queryset(self):
        try:
            return Issue.objects.filter(project=self.project).prefetch_related('resources')
        except AttributeError:
            # this is needed for the swagger ui
            return Issue.objects.none()


class ProjectSnapshotViewSet(ProjectNestedViewSetMixin, CreateModelMixin, RetrieveModelMixin,
                             UpdateModelMixin, ListModelMixin, GenericViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = ProjectSnapshotSerializer

    def get_queryset(self):
        try:
            return self.project.snapshots.all()
        except AttributeError:
            # this is needed for the swagger ui
            return Snapshot.objects.none()


class ProjectValueViewSet(ProjectNestedViewSetMixin, ModelViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = ProjectValueSerializer

    filter_backends = (ValueFilterBackend, DjangoFilterBackend)
    filterset_fields = (
        'attribute__path',
        'option',
        'option__path',
    )

    def get_queryset(self):
        try:
            return self.project.values.filter(snapshot=None)
        except AttributeError:
            # this is needed for the swagger ui
            return Value.objects.none()

    @action(detail=True, methods=['DELETE'],
            permission_classes=(HasModelPermission | HasObjectPermission, ))
    def set(self, request, parent_lookup_project, pk=None):
        # delete all values for questions in questionset collections with the attribute
        # for this value and the same set_prefix and set_index
        value = self.get_object()
        value.delete()

        attributes = Question.objects.filter_by_catalog(self.project.catalog) \
                                     .filter(questionset__is_collection=True, questionset__attribute=value.attribute) \
                                     .values_list('attribute', flat=True)
        values = self.get_queryset().filter(attribute__in=attributes, set_prefix=value.set_prefix, set_index=value.set_index)
        values.delete()

        return Response(status=204)

    @action(detail=True, methods=['GET', 'POST'],
            permission_classes=(HasModelPermission | HasObjectPermission, ))
    def file(self, request, parent_lookup_project, pk=None):
        value = self.get_object()

        if request.method == 'POST':
            value.file = request.FILES.get('file')

            # check if the project is reached
            if value.file and value.file.size + value.project.file_size > human2bytes(settings.PROJECT_FILE_QUOTA):
                raise serializers.ValidationError({
                    'value': [_('You reached the file quota for this project.')]
                })

            value.save()
            serializer = self.get_serializer(value)
            return Response(serializer.data)

        else:
            if value.file:
                return return_file_response(value.file.name, value.file_type)

        # if it didn't work return 404
        raise NotFound()


class ProjectQuestionSetViewSet(ProjectNestedViewSetMixin, RetrieveModelMixin, GenericViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = QuestionSetSerializer

    def get_queryset(self):
        try:
            return QuestionSet.objects.order_by_catalog(self.project.catalog).select_related('section', 'section__catalog')
        except AttributeError:
            # this is needed for the swagger ui
            return QuestionSet.objects.none()

    def dispatch(self, *args, **kwargs):
        response = super().dispatch(*args, **kwargs)

        if response.status_code == 200 and kwargs.get('pk'):
            try:
                continuation = Continuation.objects.get(project=self.project, user=self.request.user)
            except Continuation.DoesNotExist:
                continuation = Continuation(project=self.project, user=self.request.user)

            continuation.questionset_id = kwargs.get('pk')
            continuation.save()

        return response

    def retrieve(self, request, *args, **kwargs):
        questionset = self.get_object()
        conditions = questionset.conditions.select_related('source', 'target_option')

        values = self.project.values.filter(snapshot=None).select_related('attribute', 'option')

        if check_conditions(conditions, values):
            serializer = self.get_serializer(questionset)
            return Response(serializer.data)
        else:
            if request.GET.get('back') == 'true' and questionset.prev is not None:
                url = reverse('v1-projects:project-questionset-detail', args=[self.project.id, questionset.prev]) + '?back=true'
                return HttpResponseRedirect(url, status=303)
            elif questionset.next is not None:
                url = reverse('v1-projects:project-questionset-detail', args=[self.project.id, questionset.next])
                return HttpResponseRedirect(url, status=303)
            else:
                # indicate end of catalog
                return Response(status=204)

    @action(detail=False, url_path='continue', permission_classes=(HasModelPermission | HasObjectPermission, ))
    def get_continue(self, request, pk=None, parent_lookup_project=None):
        try:
            continuation = Continuation.objects.get(project=self.project, user=self.request.user)

            if continuation.questionset.section.catalog == self.project.catalog:
                questionset = continuation.questionset
            else:
                questionset = self.get_queryset().first()

        except Continuation.DoesNotExist:
            questionset = self.get_queryset().first()

        serializer = self.get_serializer(questionset)
        return Response(serializer.data)


class MembershipViewSet(ReadOnlyModelViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = MembershipSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'user',
        'user__username',
        'role'
    )

    def get_queryset(self):
        return Membership.objects.filter_user(self.request.user)

    def get_detail_permission_object(self, obj):
        return obj.project


class IntegrationViewSet(ReadOnlyModelViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = IntegrationSerializer

    filter_backends = (DjangoFilterBackend, )
    filterset_fields = (
        'project',
        'provider_key'
    )

    def get_queryset(self):
        return Integration.objects.filter_user(self.request.user)

    def get_detail_permission_object(self, obj):
        return obj.project


class IssueViewSet(ReadOnlyModelViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = IssueSerializer

    filter_backends = (DjangoFilterBackend, )
    filterset_fields = (
        'task',
        'task__uri',
        'status'
    )

    def get_queryset(self):
        return Issue.objects.filter_user(self.request.user).prefetch_related('resources')

    def get_detail_permission_object(self, obj):
        return obj.project


class SnapshotViewSet(ReadOnlyModelViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = SnapshotSerializer

    filter_backends = (DjangoFilterBackend,)
    filterset_fields = (
        'title',
        'project'
    )

    def get_queryset(self):
        return Snapshot.objects.filter_user(self.request.user)

    def get_detail_permission_object(self, obj):
        return obj.project


class ValueViewSet(ReadOnlyModelViewSet):
    permission_classes = (HasModelPermission | HasObjectPermission, )
    serializer_class = ValueSerializer

    filter_backends = (SnapshotFilterBackend, DjangoFilterBackend)
    filterset_fields = (
        'project',
        'attribute',
        'attribute__path',
        'option',
        'option__path',
    )

    def get_queryset(self):
        return Value.objects.filter_user(self.request.user)

    def get_detail_permission_object(self, obj):
        return obj.project

    @action(detail=True, permission_classes=(HasModelPermission | HasObjectPermission, ))
    def file(self, request, pk=None):
        value = self.get_object()

        if value.file:
            return return_file_response(value.file.name, value.file_type)

        # if it didn't work return 404
        raise NotFound()



from .integration import (IntegrationCreateView, IntegrationDeleteView,
                          IntegrationUpdateView, IntegrationWebhookView)
from .invite import InviteDeleteView
from .issue import IssueDetailView, IssueSendView, IssueUpdateView
from .membership import (MembershipCreateView, MembershipDeleteView,
                         MembershipUpdateView)
from .project import (ProjectCancelView, ProjectDeleteView, ProjectDetailView,
                      ProjectErrorView, ProjectExportView, ProjectJoinView,
                      ProjectLeaveView, ProjectQuestionsView, ProjectsView,
                      SiteProjectsView)
from .project_answers import ProjectAnswersExportView, ProjectAnswersView
from .project_create import ProjectCreateImportView, ProjectCreateView
from .project_update import (ProjectUpdateCatalogView, ProjectUpdateImportView,
                             ProjectUpdateInformationView,
                             ProjectUpdateParentView, ProjectUpdateTasksView,
                             ProjectUpdateView, ProjectUpdateViewsView)
from .project_view import ProjectViewExportView, ProjectViewView
from .snapshot import (SnapshotCreateView, SnapshotRollbackView,
                       SnapshotUpdateView)



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from rdmo.conditions.models import Condition
from rdmo.core.models import TranslationMixin
from rdmo.core.plugins import get_plugin
from rdmo.core.utils import copy_model, join_url


class OptionSet(models.Model):

    uri = models.URLField(
        max_length=640, blank=True,
        verbose_name=_('URI'),
        help_text=_('The Uniform Resource Identifier of this option set (auto-generated).')
    )
    uri_prefix = models.URLField(
        max_length=256,
        verbose_name=_('URI Prefix'),
        help_text=_('The prefix for the URI of this option set.')
    )
    key = models.SlugField(
        max_length=128, blank=True,
        verbose_name=_('Key'),
        help_text=_('The internal identifier of this option set.')
    )
    comment = models.TextField(
        blank=True,
        verbose_name=_('Comment'),
        help_text=_('Additional internal information about this option set.')
    )
    locked = models.BooleanField(
        default=False,
        verbose_name=_('Locked'),
        help_text=_('Designates whether this option set (and it\'s options) can be changed.')
    )
    order = models.IntegerField(
        default=0,
        verbose_name=_('Order'),
        help_text=_('The position of this option set in lists.')
    )
    provider_key = models.SlugField(
        max_length=128, blank=True,
        verbose_name=_('Provider'),
        help_text=_('The provider for this optionset. If set, it will create dynamic options for this optionset.')
    )
    conditions = models.ManyToManyField(
        Condition, blank=True, related_name='optionsets',
        verbose_name=_('Conditions'),
        help_text=_('The list of conditions evaluated for this option set.')
    )

    class Meta:
        ordering = ('uri', )
        verbose_name = _('Option set')
        verbose_name_plural = _('Option sets')

    def __str__(self):
        return self.key

    def save(self, *args, **kwargs):
        self.uri = self.build_uri(self.uri_prefix, self.key)
        super().save(*args, **kwargs)

        for option in self.options.all():
            option.save()

    def copy(self, uri_prefix, key):
        optionset = copy_model(self, uri_prefix=uri_prefix, key=key)

        # copy m2m fields
        optionset.conditions.set(self.conditions.all())

        # copy children
        for option in self.options.all():
            option.copy(uri_prefix, option.key, optionset=optionset)

        return optionset

    @property
    def label(self):
        return self.uri

    @property
    def provider(self):
        return get_plugin('OPTIONSET_PROVIDERS', self.provider_key)

    @property
    def has_provider(self):
        return self.provider is not None

    @property
    def has_search(self):
        return self.has_provider and self.provider.search

    @property
    def has_conditions(self):
        return self.conditions.exists()

    @property
    def is_locked(self):
        return self.locked

    @classmethod
    def build_uri(cls, uri_prefix, key):
        assert key
        return join_url(uri_prefix or settings.DEFAULT_URI_PREFIX, '/options/', key)


class Option(models.Model, TranslationMixin):

    uri = models.URLField(
        max_length=640, blank=True,
        verbose_name=_('URI'),
        help_text=_('The Uniform Resource Identifier of this option (auto-generated).')
    )
    uri_prefix = models.URLField(
        max_length=256,
        verbose_name=_('URI Prefix'),
        help_text=_('The prefix for the URI of this option.')
    )
    key = models.SlugField(
        max_length=128, blank=True,
        verbose_name=_('Key'),
        help_text=_('The internal identifier of this option.')
    )
    path = models.SlugField(
        max_length=512, blank=True,
        verbose_name=_('Path'),
        help_text=_('The path part of the URI for this option (auto-generated).')
    )
    comment = models.TextField(
        blank=True,
        verbose_name=_('Comment'),
        help_text=_('Additional internal information about this option.')
    )
    locked = models.BooleanField(
        default=False,
        verbose_name=_('Locked'),
        help_text=_('Designates whether this option can be changed.')
    )
    optionset = models.ForeignKey(
        'OptionSet', on_delete=models.CASCADE, related_name='options',
        verbose_name=_('Option set'),
        help_text=_('The option set this option belongs to.')
    )
    order = models.IntegerField(
        default=0,
        verbose_name=_('Order'),
        help_text=_('Position in lists.')
    )
    text_lang1 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Text (primary)'),
        help_text=_('The text for this option in the primary language.')
    )
    text_lang2 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Text (secondary)'),
        help_text=_('The text for this option in the secondary language.')
    )
    text_lang3 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Text (tertiary)'),
        help_text=_('The text for this option in the tertiary language.')
    )
    text_lang4 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Text (quaternary)'),
        help_text=_('The text for this option in the quaternary language.')
    )
    text_lang5 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Text (quinary)'),
        help_text=_('The text for this option in the quinary language.')
    )
    additional_input = models.BooleanField(
        default=False,
        verbose_name=_('Additional input'),
        help_text=_('Designates whether an additional input is possible for this option.')
    )

    class Meta:
        ordering = ('optionset__order', 'optionset__key', 'order', 'key')
        verbose_name = _('Option')
        verbose_name_plural = _('Options')

    def __str__(self):
        return self.path

    def save(self, *args, **kwargs):
        self.path = self.build_path(self.key, self.optionset)
        self.uri = self.build_uri(self.uri_prefix, self.path)
        super().save(*args, **kwargs)

    def copy(self, uri_prefix, key, optionset=None):
        return copy_model(self, uri_prefix=uri_prefix, key=key, optionset=optionset or self.optionset)

    @property
    def parent_fields(self):
        return ('optionset', )

    @property
    def text(self):
        return self.trans('text')

    @property
    def label(self):
        return '%s ("%s")' % (self.uri, self.text)

    @property
    def is_locked(self):
        return self.locked or self.optionset.locked

    @classmethod
    def build_path(cls, key, optionset):
        assert key
        assert optionset
        return '%s/%s' % (optionset.key, key) if (optionset and key) else None

    @classmethod
    def build_uri(cls, uri_prefix, path):
        assert path
        return join_url(uri_prefix or settings.DEFAULT_URI_PREFIX, '/options/', path)



import pytest

from rdmo.core.utils import join_url, sanitize_url

urls = (
    ('', ''),
    ('/', ''),
    ('foo', 'foo'),
    ('/foo', '/foo'),
    ('/foo/', '/foo/'),
    ('foo/bar', 'foo/bar'),
    ('foo/bar/', 'foo/bar/'),
    ('/foo/bar', '/foo/bar'),
    ('/foo/bar/', '/foo/bar/'),
    (1, ''),
)


@pytest.mark.parametrize("url,sanitized_url", urls)
def test_sanitize_url(url, sanitized_url):
    assert sanitize_url(url) == sanitized_url


def test_join_url():
    assert join_url('https://example.com//', '/terms', 'foo') == 'https://example.com/terms/foo'



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-08-03 14:30
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0014_is_attribute'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='attributeentity',
            options={'ordering': ('label',), 'verbose_name': 'AttributeEntity', 'verbose_name_plural': 'AttributeEntities'},
        ),
        migrations.RenameField(
            model_name='attributeentity',
            old_name='full_title',
            new_name='label',
        ),
    ]



from django.conf import settings
from django.contrib.sites.models import Site
from django.db import models
from django.utils.translation import gettext_lazy as _


class Overlay(models.Model):

    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.CASCADE, related_name='overlays',
        verbose_name=_('User'),
        help_text=_('The user for this overlay.')
    )
    site = models.ForeignKey(
        Site, on_delete=models.SET_NULL, null=True,
        verbose_name=_('Site'),
        help_text=_('The site for this overlay.')
    )
    url_name = models.SlugField(
        max_length=128,
        verbose_name=_('Url name'),
        help_text=_('The url_name for this overlay.')
    )
    current = models.SlugField(
        max_length=128, blank=True,
        verbose_name=_('Current'),
        help_text=_('The current state for this overlay.')
    )

    class Meta:
        ordering = ('user', 'url_name')
        verbose_name = _('Overlay')
        verbose_name_plural = _('Overlays')

    def __str__(self):
        return '{} / {}'.format(self.user, self.url_name)



from django.conf import settings
from django.contrib.auth.models import Group
from django.contrib.sites.models import Site
from django.db import models
from django.utils.translation import gettext_lazy as _

from rdmo.core.models import Model, TranslationMixin
from rdmo.core.utils import copy_model, get_language_fields, join_url

from ..managers import CatalogManager


class Catalog(Model, TranslationMixin):

    objects = CatalogManager()

    uri = models.URLField(
        max_length=640, blank=True,
        verbose_name=_('URI'),
        help_text=_('The Uniform Resource Identifier of this catalog (auto-generated).')
    )
    uri_prefix = models.URLField(
        max_length=256,
        verbose_name=_('URI Prefix'),
        help_text=_('The prefix for the URI of this catalog.')
    )
    key = models.SlugField(
        max_length=128, blank=True,
        verbose_name=_('Key'),
        help_text=_('The internal identifier of this catalog.')
    )
    comment = models.TextField(
        blank=True,
        verbose_name=_('Comment'),
        help_text=_('Additional internal information about this catalog.')
    )
    locked = models.BooleanField(
        default=False,
        verbose_name=_('Locked'),
        help_text=_('Designates whether this catalog (and it\'s sections, question sets and questions) can be changed.')
    )
    order = models.IntegerField(
        default=0,
        verbose_name=_('Order'),
        help_text=_('The position of this catalog in lists.')
    )
    sites = models.ManyToManyField(
        Site, blank=True,
        verbose_name=_('Sites'),
        help_text=_('The sites this catalog belongs to (in a multi site setup).')
    )
    groups = models.ManyToManyField(
        Group, blank=True,
        verbose_name=_('Group'),
        help_text=_('The groups for which this catalog is active.')
    )
    title_lang1 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (primary)'),
        help_text=_('The title for this catalog in the primary language.')
    )
    title_lang2 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (secondary)'),
        help_text=_('The title for this catalog in the secondary language.')
    )
    title_lang3 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (tertiary)'),
        help_text=_('The title for this catalog in the tertiary language.')
    )
    title_lang4 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (quaternary)'),
        help_text=_('The title for this catalog in the quaternary language.')
    )
    title_lang5 = models.CharField(
        max_length=256, blank=True,
        verbose_name=_('Title (quinary)'),
        help_text=_('The title for this catalog in the quinary language.')
    )
    help_lang1 = models.TextField(
        blank=True,
        verbose_name=_('Help (primary)'),
        help_text=_('The help text for this catalog in the primary language.')
    )
    help_lang2 = models.TextField(
        blank=True,
        verbose_name=_('Help (secondary)'),
        help_text=_('The help text for this catalog in the secondary language.')
    )
    help_lang3 = models.TextField(
        blank=True,
        verbose_name=_('Help (tertiary)'),
        help_text=_('The help text for this catalog in the tertiary language.')
    )
    help_lang4 = models.TextField(
        blank=True,
        verbose_name=_('Help (quaternary)'),
        help_text=_('The help text for this catalog in the quaternary language.')
    )
    help_lang5 = models.TextField(
        blank=True,
        verbose_name=_('Help (quinary)'),
        help_text=_('The help text for this catalog in the quinary language.')
    )
    available = models.BooleanField(
        default=True,
        verbose_name=_('Available'),
        help_text=_('Designates whether this catalog is generally available for projects.')
    )

    class Meta:
        ordering = ('order',)
        verbose_name = _('Catalog')
        verbose_name_plural = _('Catalogs')

    def __str__(self):
        return self.key

    def save(self, *args, **kwargs):
        self.uri = self.build_uri(self.uri_prefix, self.key)
        super().save(*args, **kwargs)

        for section in self.sections.all():
            section.save()

    def copy(self, uri_prefix, key):
        # create a new title
        kwargs = {}
        for field in get_language_fields('title'):
            kwargs[field] = getattr(self, field) + '*'

        # copy instance
        catalog = copy_model(self, uri_prefix=uri_prefix, key=key, **kwargs)

        # copy m2m fields
        catalog.sites.set(self.sites.all())
        catalog.groups.set(self.groups.all())

        # copy children
        for section in self.sections.all():
            section.copy(uri_prefix, section.key, catalog=catalog)

        return catalog

    @property
    def title(self):
        return self.trans('title')

    @property
    def help(self):
        return self.trans('help')

    @property
    def is_locked(self):
        return self.locked

    @classmethod
    def build_uri(cls, uri_prefix, key):
        assert key
        return join_url(uri_prefix or settings.DEFAULT_URI_PREFIX, '/questions/', key)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-07-26 15:00
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='condition',
            name='content_type',
        ),
        migrations.RemoveField(
            model_name='condition',
            name='object_id',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-21 13:14
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0031_rename_attribute_entity_to_attribute'),
    ]

    operations = [
        migrations.AlterField(
            model_name='question',
            name='attribute',
            field=models.ForeignKey(blank=True, help_text='The attribute this question belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Attribute', verbose_name='Attribute'),
        ),
        migrations.AlterField(
            model_name='questionset',
            name='attribute',
            field=models.ForeignKey(blank=True, help_text='The attribute this questionset belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Attribute', verbose_name='Attribute'),
        ),
    ]



import pytest
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import QuestionSet, Section
from ..serializers.v1 import QuestionSetSerializer
from ..validators import QuestionSetLockedValidator


def test_create(db):
    QuestionSetLockedValidator()({
        'section': Section.objects.first(),
        'locked': False
    })


def test_create_locked(db):
    QuestionSetLockedValidator()({
        'section': Section.objects.first(),
        'locked': True
    })


def test_update(db):
    questionset = QuestionSet.objects.first()

    QuestionSetLockedValidator(questionset)({
        'section': questionset.section,
        'locked': False
    })


def test_update_error(db):
    questionset = QuestionSet.objects.first()
    questionset.locked = True
    questionset.save()

    with pytest.raises(ValidationError):
        QuestionSetLockedValidator(questionset)({
            'section': questionset.section,
            'locked': True
        })


def test_update_parent_error(db):
    questionset = QuestionSet.objects.first()
    questionset.section.locked = True
    questionset.section.save()

    with pytest.raises(ValidationError):
        QuestionSetLockedValidator(questionset)({
            'section': questionset.section,
            'locked': False
        })


def test_update_parent_parent_error(db):
    questionset = QuestionSet.objects.first()
    questionset.section.catalog.locked = True
    questionset.section.catalog.save()

    with pytest.raises(ValidationError):
        QuestionSetLockedValidator(questionset)({
            'section': questionset.section,
            'locked': False
        })


def test_update_lock(db):
    questionset = QuestionSet.objects.first()

    QuestionSetLockedValidator(questionset)({
        'section': questionset.section,
        'locked': True
    })


def test_update_unlock(db):
    questionset = QuestionSet.objects.first()
    questionset.locked = True
    questionset.save()

    QuestionSetLockedValidator(questionset)({
        'section': questionset.section,
        'locked': False
    })


def test_serializer_create(db):
    validator = QuestionSetLockedValidator()
    validator.set_context(QuestionSetSerializer())

    validator({
        'section': Section.objects.first(),
        'locked': False
    })


def test_serializer_create_locked(db):
    validator = QuestionSetLockedValidator()
    validator.set_context(QuestionSetSerializer())

    validator({
        'section': Section.objects.first(),
        'locked': True
    })


def test_serializer_update(db):
    questionset = QuestionSet.objects.first()

    validator = QuestionSetLockedValidator()
    validator.set_context(QuestionSetSerializer(instance=questionset))

    validator({
        'section': questionset.section,
        'locked': False
    })


def test_serializer_update_error(db):
    questionset = QuestionSet.objects.first()
    questionset.locked = True
    questionset.save()

    validator = QuestionSetLockedValidator()
    validator.set_context(QuestionSetSerializer(instance=questionset))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'section': questionset.section,
            'locked': True
        })


def test_serializer_update_parent_error(db):
    questionset = QuestionSet.objects.first()
    questionset.section.locked = True
    questionset.section.save()

    validator = QuestionSetLockedValidator()
    validator.set_context(QuestionSetSerializer(instance=questionset))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'section': questionset.section,
            'locked': True
        })


def test_serializer_update_parent_parent_error(db):
    questionset = QuestionSet.objects.first()
    questionset.section.catalog.locked = True
    questionset.section.catalog.save()

    validator = QuestionSetLockedValidator()
    validator.set_context(QuestionSetSerializer(instance=questionset))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'section': questionset.section,
            'locked': True
        })


def test_serializer_update_lock(db):
    questionset = QuestionSet.objects.first()

    validator = QuestionSetLockedValidator()
    validator.set_context(QuestionSetSerializer(instance=questionset))

    validator({
        'section': questionset.section,
        'locked': True
    })


def test_serializer_update_unlock(db):
    questionset = QuestionSet.objects.first()
    questionset.locked = True
    questionset.save()

    validator = QuestionSetLockedValidator()
    validator.set_context(QuestionSetSerializer(instance=questionset))

    validator({
        'section': questionset.section,
        'locked': False
    })



from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.contrib.sites.models import Site
from rest_framework import serializers

from rdmo.projects.models import Membership

from ..models import Role


class SiteSerializer(serializers.ModelSerializer):

    class Meta:
        model = Site
        fields = (
            'id',
            'name',
            'domain'
        )


class GroupSerializer(serializers.ModelSerializer):

    class Meta:
        model = Group
        fields = (
            'id',
            'name'
        )


class RoleSerializer(serializers.ModelSerializer):

    member = SiteSerializer(many=True)
    manager = SiteSerializer(many=True)

    class Meta:
        model = Role
        fields = (
            'id',
            'member',
            'manager'
        )


class MembershipSerializer(serializers.ModelSerializer):

    class Meta:
        model = Membership
        fields = (
            'id',
            'project',
            'role'
        )


class UserSerializer(serializers.ModelSerializer):

    groups = GroupSerializer(many=True)
    role = RoleSerializer()
    memberships = MembershipSerializer(many=True)

    class Meta:
        model = get_user_model()
        fields = [
            'id',
            'groups',
            'role',
            'memberships'
        ]
        if settings.USER_API:
            fields += [
                'username',
                'first_name',
                'last_name',
                'email'
            ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def run_data_migration(apps, schema_editor):
    QuestionSet = apps.get_model('questions', 'QuestionSet')
    Question = apps.get_model('questions', 'Question')
    VerboseName = apps.get_model('domain', 'VerboseName')
    Range = apps.get_model('domain', 'Range')

    for questionset in QuestionSet.objects.exclude(attribute_entity=None):
        try:
            verbose_name = questionset.attribute_entity.verbosename

            questionset.verbose_name_en = verbose_name.name_en
            questionset.verbose_name_de = verbose_name.name_de
            questionset.verbose_name_plural_en = verbose_name.name_plural_en
            questionset.verbose_name_plural_de = verbose_name.name_plural_de
            questionset.save()

        except VerboseName.DoesNotExist:
            pass

    for question in Question.objects.all():
        try:
            verbose_name = question.attribute_entity.verbosename

            question.verbose_name_en = verbose_name.name_en
            question.verbose_name_de = verbose_name.name_de
            question.verbose_name_plural_en = verbose_name.name_plural_en
            question.verbose_name_plural_de = verbose_name.name_plural_de
            question.save()

        except VerboseName.DoesNotExist:
            pass

        try:
            range = question.attribute_entity.attribute.range

            question.minimum = range.minimum
            question.maximum = range.maximum
            question.step = range.step
            question.save()

        except Range.DoesNotExist:
            pass


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0029_verbose_name_and_range'),
        ('domain', '0035_remove_is_collection_and_parent_collection')
    ]

    operations = [
        migrations.RunPython(run_data_migration),
    ]



from django.conf import settings

auth_app, auth_model = settings.AUTH_USER_MODEL.lower().split('.')

user_view_permission = (
  auth_app,
  auth_model,
  'view_{}'.format(auth_model)
)

GROUPS = (
  ('editor', (
      ('domain', 'attribute', 'add_attribute'),
      ('domain', 'attribute', 'change_attribute'),
      ('domain', 'attribute', 'delete_attribute'),
      ('domain', 'attribute', 'view_attribute'),
      ('options', 'option', 'add_option'),
      ('options', 'option', 'change_option'),
      ('options', 'option', 'delete_option'),
      ('options', 'option', 'view_option'),
      ('options', 'optionset', 'add_optionset'),
      ('options', 'optionset', 'change_optionset'),
      ('options', 'optionset', 'delete_optionset'),
      ('options', 'optionset', 'view_optionset'),
      ('conditions', 'condition', 'add_condition'),
      ('conditions', 'condition', 'change_condition'),
      ('conditions', 'condition', 'delete_condition'),
      ('conditions', 'condition', 'view_condition'),
      ('questions', 'section', 'add_section'),
      ('questions', 'section', 'change_section'),
      ('questions', 'section', 'delete_section'),
      ('questions', 'section', 'view_section'),
      ('questions', 'catalog', 'add_catalog'),
      ('questions', 'catalog', 'change_catalog'),
      ('questions', 'catalog', 'delete_catalog'),
      ('questions', 'catalog', 'view_catalog'),
      ('questions', 'questionset', 'add_questionset'),
      ('questions', 'questionset', 'change_questionset'),
      ('questions', 'questionset', 'delete_questionset'),
      ('questions', 'questionset', 'view_questionset'),
      ('questions', 'question', 'add_question'),
      ('questions', 'question', 'change_question'),
      ('questions', 'question', 'delete_question'),
      ('questions', 'question', 'view_question'),
      ('tasks', 'task', 'add_task'),
      ('tasks', 'task', 'change_task'),
      ('tasks', 'task', 'delete_task'),
      ('tasks', 'task', 'view_task'),
      ('views', 'view', 'add_view'),
      ('views', 'view', 'change_view'),
      ('views', 'view', 'delete_view'),
      ('views', 'view', 'view_view'),
      ('sites', 'site', 'view_site'),
      ('auth', 'group', 'view_group'),
  )),
  ('reviewer', (
      ('domain', 'attribute', 'view_attribute'),
      ('options', 'option', 'view_option'),
      ('options', 'optionset', 'view_optionset'),
      ('conditions', 'condition', 'view_condition'),
      ('questions', 'catalog', 'view_catalog'),
      ('questions', 'section', 'view_section'),
      ('questions', 'questionset', 'view_questionset'),
      ('questions', 'question', 'view_question'),
      ('tasks', 'task', 'view_task'),
      ('views', 'view', 'view_view'),
      ('sites', 'site', 'view_site'),
      ('auth', 'group', 'view_group'),
  )),
  ('api', (
      user_view_permission,
      ('domain', 'attribute', 'add_attribute'),
      ('domain', 'attribute', 'change_attribute'),
      ('domain', 'attribute', 'delete_attribute'),
      ('domain', 'attribute', 'view_attribute'),
      ('options', 'option', 'add_option'),
      ('options', 'option', 'change_option'),
      ('options', 'option', 'delete_option'),
      ('options', 'option', 'view_option'),
      ('options', 'optionset', 'add_optionset'),
      ('options', 'optionset', 'change_optionset'),
      ('options', 'optionset', 'delete_optionset'),
      ('options', 'optionset', 'view_optionset'),
      ('conditions', 'condition', 'add_condition'),
      ('conditions', 'condition', 'change_condition'),
      ('conditions', 'condition', 'delete_condition'),
      ('conditions', 'condition', 'view_condition'),
      ('questions', 'section', 'add_section'),
      ('questions', 'section', 'change_section'),
      ('questions', 'section', 'delete_section'),
      ('questions', 'section', 'view_section'),
      ('questions', 'catalog', 'add_catalog'),
      ('questions', 'catalog', 'change_catalog'),
      ('questions', 'catalog', 'delete_catalog'),
      ('questions', 'catalog', 'view_catalog'),
      ('questions', 'questionset', 'add_questionset'),
      ('questions', 'questionset', 'change_questionset'),
      ('questions', 'questionset', 'delete_questionset'),
      ('questions', 'questionset', 'view_questionset'),
      ('questions', 'question', 'add_question'),
      ('questions', 'question', 'change_question'),
      ('questions', 'question', 'delete_question'),
      ('questions', 'question', 'view_question'),
      ('tasks', 'task', 'add_task'),
      ('tasks', 'task', 'change_task'),
      ('tasks', 'task', 'delete_task'),
      ('tasks', 'task', 'view_task'),
      ('views', 'view', 'add_view'),
      ('views', 'view', 'change_view'),
      ('views', 'view', 'delete_view'),
      ('views', 'view', 'view_view'),
      ('projects', 'project', 'add_project'),
      ('projects', 'project', 'change_project'),
      ('projects', 'project', 'delete_project'),
      ('projects', 'project', 'view_project'),
      ('projects', 'snapshot', 'add_snapshot'),
      ('projects', 'snapshot', 'change_snapshot'),
      ('projects', 'snapshot', 'delete_snapshot'),
      ('projects', 'snapshot', 'view_snapshot'),
      ('projects', 'value', 'add_value'),
      ('projects', 'value', 'change_value'),
      ('projects', 'value', 'delete_value'),
      ('projects', 'value', 'view_value'),
      ('projects', 'membership', 'add_membership'),
      ('projects', 'membership', 'change_membership'),
      ('projects', 'membership', 'delete_membership'),
      ('projects', 'membership', 'view_membership'),
      ('projects', 'issue', 'add_issue'),
      ('projects', 'issue', 'change_issue'),
      ('projects', 'issue', 'delete_issue'),
      ('projects', 'issue', 'view_issue'),
      ('projects', 'integration', 'add_integration'),
      ('projects', 'integration', 'change_integration'),
      ('projects', 'integration', 'delete_integration'),
      ('projects', 'integration', 'view_integration'),
  ))
)



# Generated by Django 2.2.16 on 2020-10-28 10:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0042_allow_site_null'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='catalog',
            field=models.ForeignKey(help_text='The catalog which will be used for this project.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='projects', to='questions.Catalog', verbose_name='Catalog'),
        ),
    ]



from django.urls import re_path

from ..views import CatalogExportView, CatalogsView

urlpatterns = [
    re_path(r'^catalogs/(?P<pk>[0-9]+)/export/(?P<format>[a-z]+)/$', CatalogExportView.as_view(), name='questions_catalog_export'),
    re_path(r'^catalogs/', CatalogsView.as_view(), name='catalogs'),
]



# Generated by Django 2.2.13 on 2020-08-24 11:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0030_available'),
    ]

    operations = [
        migrations.AlterField(
            model_name='task',
            name='conditions',
            field=models.ManyToManyField(blank=True, help_text='The list of conditions evaluated for this task.', related_name='tasks', to='conditions.Condition', verbose_name='Conditions'),
        ),
        migrations.AlterField(
            model_name='task',
            name='end_attribute',
            field=models.ForeignKey(blank=True, help_text='The attribute that is setting the end date for this task (optional, if no end date attribute is given, the start date attribute sets also the end date).', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='tasks_as_end', to='domain.Attribute', verbose_name='End date attribute'),
        ),
        migrations.AlterField(
            model_name='task',
            name='start_attribute',
            field=models.ForeignKey(blank=True, help_text='The attribute that is setting the start date for this task.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='tasks_as_start', to='domain.Attribute', verbose_name='Start date attribute'),
        ),
    ]



import logging

from rdmo.conditions.models import Condition
from rdmo.core.imports import (fetch_parents, get_foreign_field,
                               get_m2m_instances, set_common_fields,
                               set_lang_field, validate_instance)

from .models import Option, OptionSet
from .validators import (OptionLockedValidator, OptionSetLockedValidator,
                         OptionSetUniqueURIValidator, OptionUniqueURIValidator)

logger = logging.getLogger(__name__)


def import_optionset(element, save=False):
    try:
        optionset = OptionSet.objects.get(uri=element.get('uri'))
    except OptionSet.DoesNotExist:
        optionset = OptionSet()

    set_common_fields(optionset, element)

    optionset.order = element.get('order') or 0
    optionset.provider_key = element.get('provider_key') or ''

    conditions = get_m2m_instances(optionset, element.get('conditions'), Condition)

    if save and validate_instance(optionset, OptionSetLockedValidator, OptionSetUniqueURIValidator):
        if optionset.id:
            logger.info('OptionSet created with uri %s.', element.get('uri'))
        else:
            logger.info('OptionSet %s updated.', element.get('uri'))

        optionset.save()
        optionset.conditions.set(conditions)
        optionset.imported = True

    return optionset


def import_option(element, optionset_uri=False, save=False):
    try:
        if optionset_uri is False:
            option = Option.objects.get(uri=element.get('uri'))
        else:
            option = Option.objects.get(key=element.get('key'), optionset__uri=optionset_uri)
    except Option.DoesNotExist:
        option = Option()

    set_common_fields(option, element)

    option.optionset = get_foreign_field(option, optionset_uri or element.get('optionset'), OptionSet)
    option.order = element.get('order') or 0
    option.additional_input = element.get('additional_input') or False

    set_lang_field(option, 'text', element)

    if save and validate_instance(option, OptionLockedValidator, OptionUniqueURIValidator):
        if option.id:
            logger.info('Option created with uri %s.', element.get('uri'))
        else:
            logger.info('Option %s updated.', element.get('uri'))

        option.save()
        option.imported = True

    return option


def fetch_option_parents(instances):
    return fetch_parents(OptionSet, instances)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ProjectsConfig(AppConfig):
    name = 'rdmo.projects'
    verbose_name = _('Projects')

    def ready(self):
        from . import rules
